@@ -43,6 +43,7 @@ int main(int argc, char **argv){
   progname = argv[0];
   showprgr = TRUE;
   if(getenv("HIDEPRGR")) showprgr = FALSE;
+  srand48(1978);
   if(argc < 2) usage();
   rv = 0;
   if(!strcmp(argv[1], "write")){
@@ -191,7 +192,7 @@ int runbtread(int argc, char **argv){
 /* pseudo random number generator */
 int myrand(void){
   static int cnt = 0;
-  return (rand() * rand() + (rand() >> (sizeof(int) * 4)) + (cnt++)) & 0x7FFFFFFF;
+  return (lrand48() + cnt++) & 0x7FFFFFFF;
 }
 
 
@@ -289,7 +290,7 @@ int dobtwrite(char *name, int rnum, int rnd){
     return 1;
   }
   if(rnd){
-    vlsettuning(villa, 37, 200, rnum / 45, 512);
+    vlsettuning(villa, 77, 256, rnum / 77, -1);
   } else {
     vlsettuning(villa, 101, 256, 16, 16);
   }
