@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Test cases of Cabin
- *                                                      Copyright (C) 2000-2006 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2007 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -322,7 +322,7 @@ int intpcmp(const void *ap, const void *bp){
 int myrand(void){
   static int cnt = 0;
   if(cnt == 0) srand(time(NULL));
-  return (rand() * rand() + (rand() >> (sizeof(int) * 4)) + (cnt++)) & 0x7FFFFFFF;
+  return (rand() * rand() + (rand() >> (sizeof(int) * 4)) + (cnt++)) & INT_MAX;
 }
 
 
