@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Implementation of Depot
- *                                                      Copyright (C) 2000-2006 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2007 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -43,27 +43,32 @@
 /* get the first hash value */
 #define DP_FIRSTHASH(DP_res, DP_kbuf, DP_ksiz) \
   do { \
-    int _DP_i; \
-    if((DP_ksiz) == sizeof(int)){ \
+    const unsigned char *_DP_p; \
+    int _DP_ksiz; \
+    _DP_p = (const unsigned char *)(DP_kbuf); \
+    _DP_ksiz = DP_ksiz; \
+    if((_DP_ksiz) == sizeof(int)){ \
       memcpy(&(DP_res), (DP_kbuf), sizeof(int)); \
     } else { \
       (DP_res) = 751; \
     } \
-    for(_DP_i = 0; _DP_i < (DP_ksiz); _DP_i++){ \
-      (DP_res) = (DP_res) * 31 + ((const unsigned char *)(DP_kbuf))[_DP_i]; \
+    while(_DP_ksiz--){ \
+      (DP_res) = (DP_res) * 31 + *(_DP_p)++; \
     } \
-    (DP_res) = ((DP_res) * 87767623) & 0x7FFFFFFF; \
+    (DP_res) = ((DP_res) * 87767623) & INT_MAX; \
   } while(FALSE)
 
 /* get the second hash value */
 #define DP_SECONDHASH(DP_res, DP_kbuf, DP_ksiz) \
   do { \
-    int _DP_i; \
-    (DP_res) = 19780211; \
-    for(_DP_i = (DP_ksiz) - 1; _DP_i >= 0; _DP_i--){ \
-    (DP_res) = (DP_res) * 37 + ((const unsigned char *)(DP_kbuf))[_DP_i]; \
+    const unsigned char *_DP_p; \
+    int _DP_ksiz; \
+    _DP_p = (const unsigned char *)(DP_kbuf) + DP_ksiz - 1; \
+    _DP_ksiz = DP_ksiz; \
+    for((DP_res) = 19780211; _DP_ksiz--;){ \
+      (DP_res) = (DP_res) * 37 + *(_DP_p)--; \
     } \
-    (DP_res) = ((DP_res) * 43321879) & 0x7FFFFFFF; \
+    (DP_res) = ((DP_res) * 43321879) & INT_MAX; \
   } while(FALSE)
 
 /* get the third hash value */
@@ -74,7 +79,7 @@
     for(_DP_i = (DP_ksiz) - 1; _DP_i >= 0; _DP_i--){ \
       (DP_res) = (DP_res) * 29 + ((const unsigned char *)(DP_kbuf))[_DP_i]; \
     } \
-    (DP_res) = ((DP_res) * 5157883) & 0x7FFFFFFF; \
+    (DP_res) = ((DP_res) * 5157883) & INT_MAX; \
   } while(FALSE)
 
 enum {                                   /* enumeration for a record header */
