@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Test cases of Depot
- *                                                      Copyright (C) 2000-2006 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2007 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -20,6 +20,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdarg.h>
+#include <limits.h>
 #include <time.h>
 
 #undef TRUE
@@ -285,7 +286,7 @@ void pdperror(const char *name){
 int myrand(void){
   static int cnt = 0;
   if(cnt == 0) srand(time(NULL));
-  return (rand() * rand() + (rand() >> (sizeof(int) * 4)) + (cnt++)) & 0x7FFFFFFF;
+  return (rand() * rand() + (rand() >> (sizeof(int) * 4)) + (cnt++)) & INT_MAX;
 }
 
 
