@@ -151,6 +151,8 @@ public class Curia implements ADBM {
   //----------------------------------------------------------------
   // instance fields
   //----------------------------------------------------------------
+  /** Whether to repress frequent exceptions. */
+  public boolean silent;
   /** Index of the native table for database handles. */
   private int index;
   //----------------------------------------------------------------
@@ -182,6 +184,7 @@ public class Curia implements ADBM {
    */
   public Curia(String name, int omode, int bnum, int dnum) throws CuriaException {
     synchronized(ADBM.class){
+      silent = false;
       if((index = cropen(name, omode, bnum, dnum)) == -1) throw new CuriaException(crecode());
     }
   }
@@ -231,13 +234,18 @@ public class Curia implements ADBM {
    * which means the specified value overwrites the existing one, `Curia.DKEEP', which means the
    * existing value is kept, `Curia.DCAT', which means the specified value is concatenated at
    * the end of the existing value.
+   * @return always true.  However, if the silent flag is true and replace is cancelled, false is
+   * returned instead of exception.
    * @throws CuriaException if an error occurs or replace is cancelled.
    */
-  public void put(byte[] key, byte[] val, int dmode) throws CuriaException {
+  public boolean put(byte[] key, byte[] val, int dmode) throws CuriaException {
     if(index < 0) throw new CuriaException();
     synchronized(ADBM.class){
-      if(crput(index, key, key.length, val, val.length, dmode) == 0)
+      if(crput(index, key, key.length, val, val.length, dmode) == 0){
+        if(silent && crecode() == EKEEP) return false;
         throw new CuriaException(crecode());
+      }
+      return true;
     }
   }
   /**
@@ -245,18 +253,24 @@ public class Curia implements ADBM {
    * The same as `put(key, val, Curia.DOVER)'.
    * @see #put(byte[], byte[], int)
    */
-  public void put(byte[] key, byte[] val) throws CuriaException {
-    put(key, val, DOVER);
+  public boolean put(byte[] key, byte[] val) throws CuriaException {
+    return put(key, val, DOVER);
   }
   /**
    * Delete a record.
    * @param key a byte array of a key.
+   * @return always true.  However, if the silent flag is true and no record corresponds, false
+   * is returned instead of exception.
    * @throws CuriaException if an error occurs or no record corresponds.
    */
-  public void out(byte[] key) throws CuriaException {
+  public boolean out(byte[] key) throws CuriaException {
     if(index < 0) throw new CuriaException();
     synchronized(ADBM.class){
-      if(crout(index, key, key.length) == 0) throw new CuriaException(crecode());
+      if(crout(index, key, key.length) == 0){
+        if(silent && crecode() == ENOITEM) return false;
+        throw new CuriaException(crecode());
+      }
+      return true;
     }
   }
   /**
@@ -264,7 +278,8 @@ public class Curia implements ADBM {
    * @param key a byte array of a key.
    * @param start the array index of the beginning of the value to be read.
    * @param max the max size to read with.  If it is negative, the size is unlimited.
-   * @return a byte array of the value of the corresponding record.
+   * @return a byte array of the value of the corresponding record.  If the silent flag is true
+   * and no record corresponds, `null' is returned instead of exception.
    * @throws CuriaException if an error occurs, no record corresponds, or the size of the value
    * of the corresponding record is less than the index specified by the parameter `start'.
    */
@@ -272,7 +287,10 @@ public class Curia implements ADBM {
     if(index < 0) throw new CuriaException();
     synchronized(ADBM.class){
       byte[] val = crget(index, key, key.length, start, max);
-      if(val == null) throw new CuriaException(crecode());
+      if(val == null){
+        if(silent && crecode() == ENOITEM) return null;
+        throw new CuriaException(crecode());
+      }
       return val;
     }
   }
@@ -287,7 +305,8 @@ public class Curia implements ADBM {
   /**
    * Get the size of the value of a record.
    * @param key a byte array of a key.
-   * @return the size of the value of the corresponding record.
+   * @return the size of the value of the corresponding record.  If the silent flag is true and
+   * no record corresponds, -1 is returned instead of exception.
    * @throws CuriaException if an error occurs or no record corresponds.
    * @note Because this method does not read the entity of a record, it is faster than `get'.
    */
@@ -295,7 +314,10 @@ public class Curia implements ADBM {
     if(index < 0) throw new CuriaException();
     synchronized(ADBM.class){
       int rv = crvsiz(index, key, key.length);
-      if(rv == -1) throw new CuriaException(crecode());
+      if(rv == -1){
+        if(silent && crecode() == ENOITEM) return -1;
+        throw new CuriaException(crecode());
+      }
       return rv;
     }
   }
@@ -312,7 +334,8 @@ public class Curia implements ADBM {
   }
   /**
    * Get the next key of the iterator.
-   * @return a byte array of the key of the next record.
+   * @return a byte array of the key of the next record.  If the silent flag is true and no
+   * record corresponds, `null' is returned instead of exception.
    * @throws CuriaException if an error occurs or no record corresponds.
    * @note It is possible to access every record by iteration of calling this method.
    * However, it is not assured if updating the database is occurred while the iteration.
@@ -323,7 +346,10 @@ public class Curia implements ADBM {
     if(index < 0) throw new CuriaException();
     synchronized(ADBM.class){
       byte[] val = criternext(index);
-      if(val == null) throw new CuriaException(crecode());
+      if(val == null){
+        if(silent && crecode() == ENOITEM) return null;
+        throw new CuriaException(crecode());
+      }
       return val;
     }
   }
@@ -500,13 +526,18 @@ public class Curia implements ADBM {
    * which means the specified value overwrites the existing one, `Curia.DKEEP', which means the
    * existing value is kept, `Curia.DCAT', which means the specified value is concatenated at
    * the end of the existing value.
+   * @return always true.  However, if the silent flag is true and replace is cancelled, false is
+   * returned instead of exception.
    * @throws CuriaException if an error occurs or replace is cancelled.
    */
-  public void putlob(byte[] key, byte[] val, int dmode) throws CuriaException {
+  public boolean putlob(byte[] key, byte[] val, int dmode) throws CuriaException {
     if(index < 0) throw new CuriaException();
     synchronized(ADBM.class){
-      if(crputlob(index, key, key.length, val, val.length, dmode) == 0)
+      if(crputlob(index, key, key.length, val, val.length, dmode) == 0){
+        if(silent && crecode() == EKEEP) return false;
         throw new CuriaException(crecode());
+      }
+      return true;
     }
   }
   /**
@@ -514,18 +545,24 @@ public class Curia implements ADBM {
    * The same as `putlob(key, val, Curia.DOVER)'.
    * @see #put(byte[], byte[], int)
    */
-  public void putlob(byte[] key, byte[] val) throws CuriaException {
-    putlob(key, val, DOVER);
+  public boolean putlob(byte[] key, byte[] val) throws CuriaException {
+    return putlob(key, val, DOVER);
   }
   /**
    * Delete a large object.
    * @param key a byte array of a key.
+   * @return always true.  However, if the silent flag is true and no record corresponds, false
+   * is returned instead of exception.
    * @throws CuriaException if an error occurs or no large object corresponds.
    */
-  public void outlob(byte[] key) throws CuriaException {
+  public boolean outlob(byte[] key) throws CuriaException {
     if(index < 0) throw new CuriaException();
     synchronized(ADBM.class){
-      if(croutlob(index, key, key.length) == 0) throw new CuriaException(crecode());
+      if(croutlob(index, key, key.length) == 0){
+        if(silent && crecode() == ENOITEM) return false;
+        throw new CuriaException(crecode());
+      }
+      return true;
     }
   }
   /**
@@ -533,7 +570,8 @@ public class Curia implements ADBM {
    * @param key a byte array of a key.
    * @param start the array index of the beginning of the value to be read.
    * @param max the max size to be read.  If it is negative, the size to read is unlimited.
-   * @return a byte array of the value of the corresponding large object.
+   * @return a byte array of the value of the corresponding large object.  If the silent flag is
+   * true and no record corresponds, `null' is returned instead of exception.
    * @throws CuriaException if an error occurs, no large object corresponds or the size of the
    * value of the corresponding is less than the index specified by the parameter `start'.
    */
@@ -541,7 +579,10 @@ public class Curia implements ADBM {
     if(index < 0) throw new CuriaException();
     synchronized(ADBM.class){
       byte[] val = crgetlob(index, key, key.length, start, max);
-      if(val == null) throw new CuriaException(crecode());
+      if(val == null){
+        if(silent && crecode() == ENOITEM) return null;
+        throw new CuriaException(crecode());
+      }
       return val;
     }
   }
@@ -590,7 +631,7 @@ public class Curia implements ADBM {
    * @throws CuriaException if an error occurs or replace is cancelled.
    */
   public void store(byte[] key, byte[] val, boolean replace) throws CuriaException {
-    put(key, val, replace ? DOVER : DKEEP);
+    if(!put(key, val, replace ? DOVER : DKEEP)) throw new CuriaException(EKEEP);
   }
   /**
    * Delete a record.
@@ -598,7 +639,7 @@ public class Curia implements ADBM {
    * @throws CuriaException if an error occurs or no record corresponds.
    */
   public void delete(byte[] key) throws CuriaException {
-    out(key);
+    if(!out(key)) throw new CuriaException(ENOITEM);
   }
   /**
    * Fetch a record.
@@ -607,7 +648,9 @@ public class Curia implements ADBM {
    * @throws CuriaException if an error occurs or no record corresponds.
    */
   public byte[] fetch(byte[] key) throws CuriaException {
-    return get(key, 0, -1);
+    byte[] vbuf = get(key, 0, -1);
+    if(vbuf == null) throw new CuriaException(ENOITEM);
+    return vbuf;
   }
   /**
    * Get the first key.
@@ -616,7 +659,9 @@ public class Curia implements ADBM {
    */
   public byte[] firstkey() throws CuriaException {
     iterinit();
-    return iternext();
+    byte[] kbuf = iternext();
+    if(kbuf == null) throw new CuriaException(ENOITEM);
+    return kbuf;
   }
   /**
    * Get the next key.
@@ -624,7 +669,9 @@ public class Curia implements ADBM {
    * @throws CuriaException if an error occurs or no record corresponds.
    */
   public byte[] nextkey() throws CuriaException {
-    return iternext();
+    byte[] kbuf = iternext();
+    if(kbuf == null) throw new CuriaException(ENOITEM);
+    return kbuf;
   }
   /**
    * Check whether a fatal error occured or not.
@@ -665,10 +712,10 @@ public class Curia implements ADBM {
   private static synchronized final native int crinode(int index);
   private static synchronized final native long crmtime(int index);
   private static synchronized final native int crputlob(int index, byte[] key, int ksiz,
-                                                     byte[] val, int vsiz, int dmode);
+                                                        byte[] val, int vsiz, int dmode);
   private static synchronized final native int croutlob(int index, byte[] key, int ksiz);
   private static synchronized final native byte[] crgetlob(int index, byte[] key, int ksiz,
-                                                        int start, int max);
+                                                           int start, int max);
   private static synchronized final native int crvsizlob(int index, byte[] key, int ksiz);
   private static synchronized final native int crrnumlob(int index);
   private static synchronized final native int crremove(String name);
