@@ -151,6 +151,8 @@ public class Depot implements ADBM {
   //----------------------------------------------------------------
   // instance fields
   //----------------------------------------------------------------
+  /** Whether to repress frequent exceptions. */
+  public boolean silent;
   /** Index of the native table for database handles. */
   private int index;
   //----------------------------------------------------------------
@@ -179,6 +181,7 @@ public class Depot implements ADBM {
    */
   public Depot(String name, int omode, int bnum) throws DepotException {
     synchronized(ADBM.class){
+      silent = false;
       if((index = dpopen(name, omode, bnum)) == -1) throw new DepotException(dpecode());
     }
   }
@@ -228,13 +231,18 @@ public class Depot implements ADBM {
    * which means the specified value overwrites the existing one, `Depot.DKEEP', which means the
    * existing value is kept, `Depot.DCAT', which means the specified value is concatenated at
    * the end of the existing value.
+   * @return always true.  However, if the silent flag is true and replace is cancelled, false is
+   * returned instead of exception.
    * @throws DepotException if an error occurs or replace is cancelled.
    */
-  public void put(byte[] key, byte[] val, int dmode) throws DepotException {
+  public boolean put(byte[] key, byte[] val, int dmode) throws DepotException {
     if(index < 0) throw new DepotException();
     synchronized(ADBM.class){
-      if(dpput(index, key, key.length, val, val.length, dmode) == 0)
+      if(dpput(index, key, key.length, val, val.length, dmode) == 0){
+        if(silent && dpecode() == EKEEP) return false;
         throw new DepotException(dpecode());
+      }
+      return true;
     }
   }
   /**
@@ -242,18 +250,24 @@ public class Depot implements ADBM {
    * The same as `put(key, val, Depot.DOVER)'.
    * @see #put(byte[], byte[], int)
    */
-  public void put(byte[] key, byte[] val) throws DepotException {
-    put(key, val, DOVER);
+  public boolean put(byte[] key, byte[] val) throws DepotException {
+    return put(key, val, DOVER);
   }
   /**
    * Delete a record.
    * @param key a byte array of a key.
+   * @return always true.  However, if the silent flag is true and no record corresponds, false
+   * is returned instead of exception.
    * @throws DepotException if an error occurs or no record corresponds.
    */
-  public void out(byte[] key) throws DepotException {
+  public boolean out(byte[] key) throws DepotException {
     if(index < 0) throw new DepotException();
     synchronized(ADBM.class){
-      if(dpout(index, key, key.length) == 0) throw new DepotException(dpecode());
+      if(dpout(index, key, key.length) == 0){
+        if(silent && dpecode() == ENOITEM) return false;
+        throw new DepotException(dpecode());
+      }
+      return true;
     }
   }
   /**
@@ -261,7 +275,8 @@ public class Depot implements ADBM {
    * @param key a byte array of a key.
    * @param start the array index of the beginning of the value to be read.
    * @param max the max size to read with.  If it is negative, the size is unlimited.
-   * @return a byte array of the value of the corresponding record.
+   * @return a byte array of the value of the corresponding record.  If the silent flag is true
+   * and no record corresponds, `null' is returned instead of exception.
    * @throws DepotException if an error occurs, no record corresponds, or the size of the value
    * of the corresponding record is less than the index specified by the parameter `start'.
    */
@@ -269,7 +284,10 @@ public class Depot implements ADBM {
     if(index < 0) throw new DepotException();
     synchronized(ADBM.class){
       byte[] val = dpget(index, key, key.length, start, max);
-      if(val == null) throw new DepotException(dpecode());
+      if(val == null){
+        if(silent && dpecode() == ENOITEM) return null;
+        throw new DepotException(dpecode());
+      }
       return val;
     }
   }
@@ -284,7 +302,8 @@ public class Depot implements ADBM {
   /**
    * Get the size of the value of a record.
    * @param key a byte array of a key.
-   * @return the size of the value of the corresponding record.
+   * @return the size of the value of the corresponding record.  If the silent flag is true and
+   * no record corresponds, -1 is returned instead of exception.
    * @throws DepotException if an error occurs or no record corresponds.
    * @note Because this method does not read the entity of a record, it is faster than `get'.
    */
@@ -292,7 +311,10 @@ public class Depot implements ADBM {
     if(index < 0) throw new DepotException();
     synchronized(ADBM.class){
       int rv = dpvsiz(index, key, key.length);
-      if(rv == -1) throw new DepotException(dpecode());
+      if(rv == -1){
+        if(silent && dpecode() == ENOITEM) return -1;
+        throw new DepotException(dpecode());
+      }
       return rv;
     }
   }
@@ -309,7 +331,8 @@ public class Depot implements ADBM {
   }
   /**
    * Get the next key of the iterator.
-   * @return a byte array of the key of the next record.
+   * @return a byte array of the key of the next record.  If the silent flag is true and no
+   * record corresponds, `null' is returned instead of exception.
    * @throws DepotException if an error occurs or no record corresponds.
    * @note It is possible to access every record by iteration of calling this method.
    * However, it is not assured if updating the database is occurred while the iteration.
@@ -320,7 +343,10 @@ public class Depot implements ADBM {
     if(index < 0) throw new DepotException();
     synchronized(ADBM.class){
       byte[] val = dpiternext(index);
-      if(val == null) throw new DepotException(dpecode());
+      if(val == null){
+        if(silent && dpecode() == ENOITEM) return null;
+        throw new DepotException(dpecode());
+      }
       return val;
     }
   }
@@ -497,7 +523,7 @@ public class Depot implements ADBM {
    * @throws DepotException if an error occurs or replace is cancelled.
    */
   public void store(byte[] key, byte[] val, boolean replace) throws DepotException {
-    put(key, val, replace ? DOVER : DKEEP);
+    if(!put(key, val, replace ? DOVER : DKEEP)) throw new DepotException(EKEEP);
   }
   /**
    * Delete a record.
@@ -505,7 +531,7 @@ public class Depot implements ADBM {
    * @throws DepotException if an error occurs or no record corresponds.
    */
   public void delete(byte[] key) throws DepotException {
-    out(key);
+    if(!out(key)) throw new DepotException(ENOITEM);
   }
   /**
    * Fetch a record.
@@ -514,7 +540,9 @@ public class Depot implements ADBM {
    * @throws DepotException if an error occurs or no record corresponds.
    */
   public byte[] fetch(byte[] key) throws DepotException {
-    return get(key, 0, -1);
+    byte[] vbuf = get(key, 0, -1);
+    if(vbuf == null) throw new DepotException(ENOITEM);
+    return vbuf;
   }
   /**
    * Get the first key.
@@ -523,7 +551,9 @@ public class Depot implements ADBM {
    */
   public byte[] firstkey() throws DepotException {
     iterinit();
-    return iternext();
+    byte[] kbuf = iternext();
+    if(kbuf == null) throw new DepotException(ENOITEM);
+    return kbuf;
   }
   /**
    * Get the next key.
@@ -531,7 +561,9 @@ public class Depot implements ADBM {
    * @throws DepotException if an error occurs or no record corresponds.
    */
   public byte[] nextkey() throws DepotException {
-    return iternext();
+    byte[] kbuf = iternext();
+    if(kbuf == null) throw new DepotException(ENOITEM);
+    return kbuf;
   }
   /**
    * Check whether a fatal error occured or not.
