@@ -168,6 +168,8 @@ public class Villa implements ADBM {
   //----------------------------------------------------------------
   // instance fields
   //----------------------------------------------------------------
+  /** Whether to repress frequent exceptions. */
+  public boolean silent;
   /** Index of the native table for database handles. */
   private int index;
   /** Whether the handle has the transaction or not. */
@@ -202,6 +204,7 @@ public class Villa implements ADBM {
    */
   public Villa(String name, int omode, int cmode) throws VillaException {
     synchronized(ADBM.class){
+      silent = false;
       if((index = vlopen(name, omode, cmode)) == -1) throw new VillaException(vlecode());
     }
     tran = false;
@@ -256,14 +259,19 @@ public class Villa implements ADBM {
    * at the end of the existing value, `Villa.DDUP', which means duplication of keys is allowed
    * and the specified value is added as the last one, `Villa.DDUPR', which means duplication of
    * keys is allowed and the specified value is added as the first one.
+   * @return always true.  However, if the silent flag is true and replace is cancelled, false is
+   * returned instead of exception.
    * @throws VillaException if an error occurs or replace is cancelled.
    * @note The cursor becomes unavailable due to updating database.
    */
-  public void put(byte[] key, byte[] val, int dmode) throws VillaException {
+  public boolean put(byte[] key, byte[] val, int dmode) throws VillaException {
     if(index < 0) throw new VillaException();
     synchronized(ADBM.class){
-      if(vlput(index, key, key.length, val, val.length, dmode) == 0)
+      if(vlput(index, key, key.length, val, val.length, dmode) == 0){
+        if(silent && vlecode() == EKEEP) return false;
         throw new VillaException(vlecode());
+      }
+      return true;
     }
   }
   /**
@@ -271,8 +279,8 @@ public class Villa implements ADBM {
    * The same as `put(key, val, Villa.DOVER)'.
    * @see #put(byte[], byte[], int)
    */
-  public void put(byte[] key, byte[] val) throws VillaException {
-    put(key, val, DOVER);
+  public boolean put(byte[] key, byte[] val) throws VillaException {
+    return put(key, val, DOVER);
   }
   /**
    * Store a record composed of serializable objects.
@@ -280,23 +288,29 @@ public class Villa implements ADBM {
    * @see #put(byte[], byte[], int)
    * @note If serialization is failed, an instance of `VillaException' is thrown.
    */
-  public void putobj(Object key, Object val, int dmode) throws VillaException {
+  public boolean putobj(Object key, Object val, int dmode) throws VillaException {
     byte[] kbuf = Util.serialize(key);
     byte[] vbuf = Util.serialize(val);
     if(kbuf == null || vbuf == null) throw new VillaException();
-    put(kbuf, vbuf, dmode);
+    return put(kbuf, vbuf, dmode);
   }
   /**
    * Delete a record.
    * @param key a byte array of a key.
+   * @return always true.  However, if the silent flag is true and no record corresponds, false
+   * is returned instead of exception.
    * @throws VillaException if an error occurs or no record corresponds.
    * @note When the key of duplicated records is specified, the first record of the same key
    * is deleted.  The cursor becomes unavailable due to updating database.
    */
-  public void out(byte[] key) throws VillaException {
+  public boolean out(byte[] key) throws VillaException {
     if(index < 0) throw new VillaException();
     synchronized(ADBM.class){
-      if(vlout(index, key, key.length) == 0) throw new VillaException(vlecode());
+      if(vlout(index, key, key.length) == 0){
+        if(silent && vlecode() == ENOITEM) return false;
+        throw new VillaException(vlecode());
+      }
+      return true;
     }
   }
   /**
@@ -305,15 +319,16 @@ public class Villa implements ADBM {
    * @see #out(byte[])
    * @note If serialization is failed, an instance of `VillaException' is thrown.
    */
-  public void outobj(Object key) throws VillaException {
+  public boolean outobj(Object key) throws VillaException {
     byte[] kbuf = Util.serialize(key);
     if(kbuf == null) throw new VillaException();
-    out(kbuf);
+    return out(kbuf);
   }
   /**
    * Retrieve a record.
    * @param key a byte array of a key.
-   * @return a byte array of the value of the corresponding record.
+   * @return a byte array of the value of the corresponding record.  If the silent flag is true
+   * and no record corresponds, `null' is returned instead of exception.
    * @throws VillaException if an error occurs, no record corresponds.
    * @note When the key of duplicated records is specified, the value of the first record of
    * the same key is selected.
@@ -322,7 +337,10 @@ public class Villa implements ADBM {
     if(index < 0) throw new VillaException();
     synchronized(ADBM.class){
       byte[] val = vlget(index, key, key.length);
-      if(val == null) throw new VillaException(vlecode());
+      if(val == null){
+        if(silent && vlecode() == ENOITEM) return null;
+        throw new VillaException(vlecode());
+      }
       return val;
     }
   }
@@ -335,22 +353,28 @@ public class Villa implements ADBM {
   public Object getobj(Object key) throws VillaException {
     byte[] kbuf = Util.serialize(key);
     if(kbuf == null) throw new VillaException();
-    Object val = Util.deserialize(get(kbuf));
+    byte[] vbuf = get(kbuf);
+    if(vbuf == null) return null;
+    Object val = Util.deserialize(vbuf);
     if(val == null) throw new VillaException();
     return val;
   }
   /**
    * Get the size of the value of a record.
    * @param key a byte array of a key.
-   * @return the size of the value of the corresponding record.  If no record corresponds, -1 is
-   * returned.  If multiple records correspond, the size of the first is returned.
+   * @return the size of the value of the corresponding record.  If multiple records correspond,
+   * the size of the first is returned.  If the silent flag is true and no record corresponds, -1
+   * is returned instead of exception.
    * @throws VillaException if an error occurs.
    */
   public int vsiz(byte[] key) throws VillaException {
     if(index < 0) throw new VillaException();
     synchronized(ADBM.class){
       int rv = vlvsiz(index, key, key.length);
-      if(rv == -1) throw new VillaException(vlecode());
+      if(rv == -1){
+        if(silent && vlecode() == ENOITEM) return -1;
+        throw new VillaException(vlecode());
+      }
       return rv;
     }
   }
@@ -392,42 +416,66 @@ public class Villa implements ADBM {
   }
   /**
    * Move the cursor to the first record.
+   * @return always true.  However, if the silent flag is true and no record corresponds, false
+   * is returned instead of exception.
    * @throws VillaException if an error occurs or there is no record in the database.
    */
-  public void curfirst() throws VillaException {
+  public boolean curfirst() throws VillaException {
     if(index < 0) throw new VillaException();
     synchronized(ADBM.class){
-      if(vlcurfirst(index) == 0) throw new VillaException(vlecode());
+      if(vlcurfirst(index) == 0){
+        if(silent && vlecode() == ENOITEM) return false;
+        throw new VillaException(vlecode());
+      }
+      return true;
     }
   }
   /**
    * Move the cursor to the last record.
+   * @return always true.  However, if the silent flag is true and no record corresponds, false
+   * is returned instead of exception.
    * @throws VillaException if an error occurs or there is no record in the database.
    */
-  public void curlast() throws VillaException {
+  public boolean curlast() throws VillaException {
     if(index < 0) throw new VillaException();
     synchronized(ADBM.class){
-      if(vlcurlast(index) == 0) throw new VillaException(vlecode());
+      if(vlcurlast(index) == 0){
+        if(silent && vlecode() == ENOITEM) return false;
+        throw new VillaException(vlecode());
+      }
+      return true;
     }
   }
   /**
    * Move the cursor to the next record.
+   * @return always true.  However, if the silent flag is true and no record corresponds, false
+   * is returned instead of exception.
    * @throws VillaException if an error occurs or there is no previous record.
    */
-  public void curprev() throws VillaException {
+  public boolean curprev() throws VillaException {
     if(index < 0) throw new VillaException();
     synchronized(ADBM.class){
-      if(vlcurprev(index) == 0) throw new VillaException(vlecode());
+      if(vlcurprev(index) == 0){
+        if(silent && vlecode() == ENOITEM) return false;
+        throw new VillaException(vlecode());
+      }
+      return true;
     }
   }
   /**
    * Move the cursor to the next record.
+   * @return always true.  However, if the silent flag is true and no record corresponds, false
+   * is returned instead of exception.
    * @throws VillaException if an error occurs or there is no next record.
    */
-  public void curnext() throws VillaException {
+  public boolean curnext() throws VillaException {
     if(index < 0) throw new VillaException();
     synchronized(ADBM.class){
-      if(vlcurnext(index) == 0) throw new VillaException(vlecode());
+      if(vlcurnext(index) == 0){
+        if(silent && vlecode() == ENOITEM) return false;
+        throw new VillaException(vlecode());
+      }
+      return true;
     }
   }
   /**
@@ -438,12 +486,18 @@ public class Villa implements ADBM {
    * completely matching record does not exist, `Villa.JBACKWARD', which means that the cursor
    * is set to the last record of the same key and that the cursor is set to the previous
    * substitute if completely matching record does not exist.
+   * @return always true.  However, if the silent flag is true and no record corresponds, false
+   * is returned instead of exception.
    * @throws VillaException if an error occurs or there is no record corresponding the condition.
    */
-  public void curjump(byte[] key, int jmode) throws VillaException {
+  public boolean curjump(byte[] key, int jmode) throws VillaException {
     if(index < 0) throw new VillaException();
     synchronized(ADBM.class){
-      if(vlcurjump(index, key, key.length, jmode) == 0) throw new VillaException(vlecode());
+      if(vlcurjump(index, key, key.length, jmode) == 0){
+        if(silent && vlecode() == ENOITEM) return false;
+        throw new VillaException(vlecode());
+      }
+      return true;
     }
   }
   /**
@@ -451,8 +505,8 @@ public class Villa implements ADBM {
    * The same as `curjump(key, Villa.JFORFARD)'.
    * @see #curjump(byte[], int)
    */
-  public void curjump(byte[] key) throws VillaException {
-    curjump(key, JFORWARD);
+  public boolean curjump(byte[] key) throws VillaException {
+    return curjump(key, JFORWARD);
   }
   /**
    * Move the cursor to a position around a record composed of serializable objects.
@@ -460,20 +514,25 @@ public class Villa implements ADBM {
    * @see #curjump(byte[], int)
    * @note If serialization is failed, an instance of `VillaException' is thrown.
    */
-  public void curjumpobj(Object key, int jmode) throws VillaException {
+  public boolean curjumpobj(Object key, int jmode) throws VillaException {
     byte[] kbuf = Util.serialize(key);
     if(kbuf == null) throw new VillaException();
-    curjump(kbuf, jmode);
+    return curjump(kbuf, jmode);
   }
   /**
    * Get the key of the record where the cursor is.
+   * @return a byte array of the key of the corresponding record.  If the silent flag is true and
+   * no record corresponds, `null' is returned instead of exception.
    * @throws VillaException if an error occurs or no record corresponds to the cursor.
    */
   public byte[] curkey() throws VillaException {
     if(index < 0) throw new VillaException();
     synchronized(ADBM.class){
       byte[] val = vlcurkey(index);
-      if(val == null) throw new VillaException(vlecode());
+      if(val == null){
+        if(silent && vlecode() == ENOITEM) return null;
+        throw new VillaException(vlecode());
+      }
       return val;
     }
   }
@@ -484,19 +543,26 @@ public class Villa implements ADBM {
    * @note If serialization is failed, an instance of `VillaException' is thrown.
    */
   public Object curkeyobj() throws VillaException {
-    Object key = Util.deserialize(curkey());
+    byte[] kbuf = curkey();
+    if(kbuf == null) return null;
+    Object key = Util.deserialize(kbuf);
     if(key == null) throw new VillaException();
     return key;
   }
   /**
    * Get the value of the record where the cursor is.
+   * @return a byte array of the value of the corresponding record.  If the silent flag is true
+   * and no record corresponds, `null' is returned instead of exception.
    * @throws VillaException if an error occurs or no record corresponds to the cursor.
    */
   public byte[] curval() throws VillaException {
     if(index < 0) throw new VillaException();
     synchronized(ADBM.class){
       byte[] val = vlcurval(index);
-      if(val == null) throw new VillaException(vlecode());
+      if(val == null){
+        if(silent && vlecode() == ENOITEM) return null;
+        throw new VillaException(vlecode());
+      }
       return val;
     }
   }
@@ -507,7 +573,9 @@ public class Villa implements ADBM {
    * @note If serialization is failed, an instance of `VillaException' is thrown.
    */
   public Object curvalobj() throws VillaException {
-    Object val = Util.deserialize(curval());
+    byte[] vbuf = curval();
+    if(vbuf == null) return null;
+    Object val = Util.deserialize(vbuf);
     if(val == null) throw new VillaException();
     return val;
   }
@@ -518,13 +586,19 @@ public class Villa implements ADBM {
    * current record is overwritten, `Villa.CPBEFORE', which means that a new record is inserted
    * before the current record, `Villa.CPAFTER', which means that a new record is inserted after
    * the current record.
+   * @return always true.  However, if the silent flag is true and no record corresponds to the
+   * cursor, false is returned instead of exception.
    * @throws VillaException if an error occurs or no record corresponds to the cursor.
    * @note After insertion, the cursor is moved to the inserted record.
    */
-  public void curput(byte[] val, int cpmode) throws VillaException {
+  public boolean curput(byte[] val, int cpmode) throws VillaException {
     if(index < 0) throw new VillaException();
     synchronized(ADBM.class){
-      if(vlcurput(index, val, val.length, cpmode) == 0) throw new VillaException(vlecode());
+      if(vlcurput(index, val, val.length, cpmode) == 0){
+        if(silent && vlecode() == ENOITEM) return false;
+        throw new VillaException(vlecode());
+      }
+      return true;
     }
   }
   /**
@@ -532,28 +606,34 @@ public class Villa implements ADBM {
    * The same as `curput(val, Villa.CPCURRENT)'.
    * @see #curput(byte[], int)
    */
-  public void curput(byte[] val) throws VillaException {
-    curput(val, CPCURRENT);
+  public boolean curput(byte[] val) throws VillaException {
+    return curput(val, CPCURRENT);
   }
   /**
    * Insert a record around the cursor.
    * The same as `curput(qdbm.Util.serialize(val), cpmode)'.
    * @see #curput(byte[], int)
    */
-  public void curputobj(Object val, int cpmode) throws VillaException {
+  public boolean curputobj(Object val, int cpmode) throws VillaException {
     byte[] vbuf = Util.serialize(val);
     if(vbuf == null) throw new VillaException();
-    curput(vbuf, cpmode);
+    return curput(vbuf, cpmode);
   }
   /**
    * Delete the record where the cursor is.
+   * @return always true.  However, if the silent flag is true and no record corresponds to the
+   * cursor, false is returned instead of exception.
    * @throws VillaException if an error occurs or no record corresponds to the cursor.
    * @note After deletion, the cursor is moved to the next record if possible.
    */
-  public void curout() throws VillaException {
+  public boolean curout() throws VillaException {
     if(index < 0) throw new VillaException();
     synchronized(ADBM.class){
-      if(vlcurout(index) == 0) throw new VillaException(vlecode());
+      if(vlcurout(index) == 0){
+        if(silent && vlecode() == ENOITEM) return false;
+        throw new VillaException(vlecode());
+      }
+      return true;
     }
   }
   /**
@@ -791,7 +871,7 @@ public class Villa implements ADBM {
    * @throws VillaException if an error occurs or replace is cancelled.
    */
   public void store(byte[] key, byte[] val, boolean replace) throws VillaException {
-    put(key, val, replace ? DOVER : DKEEP);
+    if(!put(key, val, replace ? DOVER : DKEEP)) throw new VillaException(EKEEP);
   }
   /**
    * Delete a record.
@@ -799,7 +879,7 @@ public class Villa implements ADBM {
    * @throws VillaException if an error occurs or no record corresponds.
    */
   public void delete(byte[] key) throws VillaException {
-    out(key);
+    if(!out(key)) throw new VillaException(ENOITEM);
   }
   /**
    * Fetch a record.
@@ -808,7 +888,9 @@ public class Villa implements ADBM {
    * @throws VillaException if an error occurs or no record corresponds.
    */
   public byte[] fetch(byte[] key) throws VillaException {
-    return get(key);
+    byte[] vbuf = get(key);
+    if(vbuf == null) throw new VillaException(ENOITEM);
+    return vbuf;
   }
   /**
    * Get the first key.
@@ -817,7 +899,9 @@ public class Villa implements ADBM {
    */
   public byte[] firstkey() throws VillaException {
     curfirst();
-    return curkey();
+    byte[] kbuf = curkey();
+    if(kbuf == null) throw new VillaException(ENOITEM);
+    return kbuf;
   }
   /**
    * Get the next key.
@@ -826,7 +910,9 @@ public class Villa implements ADBM {
    */
   public byte[] nextkey() throws VillaException {
     curnext();
-    return curkey();
+    byte[] kbuf = curkey();
+    if(kbuf == null) throw new VillaException(ENOITEM);
+    return kbuf;
   }
   /**
    * Check whether a fatal error occured or not.
