@@ -42,7 +42,7 @@ class VillaTester extends Thread {
    * Main routine.
    * @param args specifies a name of a database file with the first element.
    */
-  public static void main(String[] args){
+  public static void main(String[] args) throws Exception {
     if(args.length != 1){
       System.err.println("usege: java qdbm.VillaTester name");
       return;
