@@ -134,7 +134,7 @@ Java_qdbm_Curia_crput(JNIEnv *env, jclass myclass, jint index, jbyteArray key, j
   int rv;
   kbuf = (*env)->GetByteArrayElements(env, key, &ick);
   vbuf = (*env)->GetByteArrayElements(env, val, &icv);
-  rv = crput(crtable[index], kbuf, ksiz, vbuf, vsiz, getcrdmode(dmode));
+  rv = crput(crtable[index], (char *)kbuf, ksiz, (char *)vbuf, vsiz, getcrdmode(dmode));
   if(ick == JNI_TRUE) (*env)->ReleaseByteArrayElements(env, key, kbuf, JNI_ABORT);
   if(icv == JNI_TRUE) (*env)->ReleaseByteArrayElements(env, val, vbuf, JNI_ABORT);
   return rv;
@@ -148,7 +148,7 @@ Java_qdbm_Curia_crout(JNIEnv *env, jclass myclass, jint index, jbyteArray key, j
   jboolean ick;
   int rv;
   kbuf = (*env)->GetByteArrayElements(env, key, &ick);
-  rv = crout(crtable[index], kbuf, ksiz);
+  rv = crout(crtable[index], (char *)kbuf, ksiz);
   if(ick == JNI_TRUE) (*env)->ReleaseByteArrayElements(env, key, kbuf, JNI_ABORT);
   return rv;
 }
@@ -164,11 +164,11 @@ Java_qdbm_Curia_crget(JNIEnv *env, jclass myclass, jint index, jbyteArray key, j
   int vsiz;
   jbyteArray vbuf;
   kbuf = (*env)->GetByteArrayElements(env, key, &ick);
-  val = crget(crtable[index], kbuf, ksiz, start, max, &vsiz);
+  val = crget(crtable[index], (char *)kbuf, ksiz, start, max, &vsiz);
   if(ick == JNI_TRUE) (*env)->ReleaseByteArrayElements(env, key, kbuf, JNI_ABORT);
   if(val){
     vbuf = (*env)->NewByteArray(env, vsiz);
-    (*env)->SetByteArrayRegion(env, vbuf, 0, vsiz, val);
+    (*env)->SetByteArrayRegion(env, vbuf, 0, vsiz, (jbyte *)val);
     free(val);
   } else {
     vbuf = NULL;
@@ -184,7 +184,7 @@ Java_qdbm_Curia_crvsiz(JNIEnv *env, jclass myclass, jint index, jbyteArray key,
   jboolean ick;
   int rv;
   kbuf = (*env)->GetByteArrayElements(env, key, &ick);
-  rv = crvsiz(crtable[index], kbuf, ksiz);
+  rv = crvsiz(crtable[index], (char *)kbuf, ksiz);
   if(ick == JNI_TRUE) (*env)->ReleaseByteArrayElements(env, key, kbuf, JNI_ABORT);
   return rv;
 }
@@ -206,7 +206,7 @@ Java_qdbm_Curia_criternext(JNIEnv *env, jclass myclass, jint index)
   val = criternext(crtable[index], &vsiz);
   if(val){
     vbuf = (*env)->NewByteArray(env, vsiz);
-    (*env)->SetByteArrayRegion(env, vbuf, 0, vsiz, val);
+    (*env)->SetByteArrayRegion(env, vbuf, 0, vsiz, (jbyte *)val);
     free(val);
   } else {
     vbuf = NULL;
@@ -324,7 +324,7 @@ Java_qdbm_Curia_crputlob(JNIEnv *env, jclass myclass, jint index, jbyteArray key
   int rv;
   kbuf = (*env)->GetByteArrayElements(env, key, &ick);
   vbuf = (*env)->GetByteArrayElements(env, val, &icv);
-  rv = crputlob(crtable[index], kbuf, ksiz, vbuf, vsiz, getcrdmode(dmode));
+  rv = crputlob(crtable[index], (char *)kbuf, ksiz, (char *)vbuf, vsiz, getcrdmode(dmode));
   if(ick == JNI_TRUE) (*env)->ReleaseByteArrayElements(env, key, kbuf, JNI_ABORT);
   if(icv == JNI_TRUE) (*env)->ReleaseByteArrayElements(env, val, vbuf, JNI_ABORT);
   return rv;
@@ -338,7 +338,7 @@ Java_qdbm_Curia_croutlob(JNIEnv *env, jclass myclass, jint index, jbyteArray key
   jboolean ick;
   int rv;
   kbuf = (*env)->GetByteArrayElements(env, key, &ick);
-  rv = croutlob(crtable[index], kbuf, ksiz);
+  rv = croutlob(crtable[index], (char *)kbuf, ksiz);
   if(ick == JNI_TRUE) (*env)->ReleaseByteArrayElements(env, key, kbuf, JNI_ABORT);
   return rv;
 }
@@ -354,11 +354,11 @@ Java_qdbm_Curia_crgetlob(JNIEnv *env, jclass myclass, jint index, jbyteArray key
   int vsiz;
   jbyteArray vbuf;
   kbuf = (*env)->GetByteArrayElements(env, key, &ick);
-  val = crgetlob(crtable[index], kbuf, ksiz, start, max, &vsiz);
+  val = crgetlob(crtable[index], (char *)kbuf, ksiz, start, max, &vsiz);
   if(ick == JNI_TRUE) (*env)->ReleaseByteArrayElements(env, key, kbuf, JNI_ABORT);
   if(val){
     vbuf = (*env)->NewByteArray(env, vsiz);
-    (*env)->SetByteArrayRegion(env, vbuf, 0, vsiz, val);
+    (*env)->SetByteArrayRegion(env, vbuf, 0, vsiz, (jbyte *)val);
     free(val);
   } else {
     vbuf = NULL;
@@ -374,7 +374,7 @@ Java_qdbm_Curia_crvsizlob(JNIEnv *env, jclass myclass, jint index, jbyteArray ke
   jboolean ick;
   int rv;
   kbuf = (*env)->GetByteArrayElements(env, key, &ick);
-  rv = crvsizlob(crtable[index], kbuf, ksiz);
+  rv = crvsizlob(crtable[index], (char *)kbuf, ksiz);
   if(ick == JNI_TRUE) (*env)->ReleaseByteArrayElements(env, key, kbuf, JNI_ABORT);
   return rv;
 }
@@ -396,7 +396,32 @@ Java_qdbm_Curia_crremove(JNIEnv *env, jclass myclass, jstring name)
   tname = (*env)->GetStringUTFChars(env, name, &ic);
   rv = crremove(tname);
   if(ic == JNI_TRUE) (*env)->ReleaseStringUTFChars(env, name, tname);
-  return rv ? 1 : 0;
+  return rv;
+}
+
+
+JNIEXPORT jbyteArray JNICALL
+Java_qdbm_Curia_crsnaffle(JNIEnv *env, jclass myclass, jstring name, jbyteArray key, jint ksiz)
+{
+  jbyte *kbuf;
+  jboolean icn, ick;
+  const char *tname;
+  char *val;
+  int vsiz;
+  jbyteArray vbuf;
+  tname = (*env)->GetStringUTFChars(env, name, &icn);
+  kbuf = (*env)->GetByteArrayElements(env, key, &ick);
+  val = crsnaffle(tname, (char *)kbuf, ksiz, &vsiz);
+  if(ick == JNI_TRUE) (*env)->ReleaseByteArrayElements(env, key, kbuf, JNI_ABORT);
+  if(icn == JNI_TRUE) (*env)->ReleaseStringUTFChars(env, name, tname);
+  if(val){
+    vbuf = (*env)->NewByteArray(env, vsiz);
+    (*env)->SetByteArrayRegion(env, vbuf, 0, vsiz, (jbyte *)val);
+    free(val);
+  } else {
+    vbuf = NULL;
+  }
+  return vbuf;
 }
 
 
