@@ -15,6 +15,7 @@
 
 
 #include "qdbm_Villa.h"
+#include "qdbm_VillaCursor.h"
 #include <depot.h>
 #include <cabin.h>
 #include <villa.h>
@@ -23,6 +24,12 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
+#if SIZEOF_VOID_P == SIZEOF_INT
+#define PTRNUM         jint
+#else
+#define PTRNUM         jlong
+#endif
+
 #define MAXOPEN 1024
 
 
@@ -149,7 +156,7 @@ Java_qdbm_Villa_vlput(JNIEnv *env, jclass myclass, jint index, jbyteArray key, j
   vlmyclass = myclass;
   kbuf = (*env)->GetByteArrayElements(env, key, &ick);
   vbuf = (*env)->GetByteArrayElements(env, val, &icv);
-  rv = vlput(vltable[index], kbuf, ksiz, vbuf, vsiz, getvldmode(dmode));
+  rv = vlput(vltable[index], (char *)kbuf, ksiz, (char *)vbuf, vsiz, getvldmode(dmode));
   if(ick == JNI_TRUE) (*env)->ReleaseByteArrayElements(env, key, kbuf, JNI_ABORT);
   if(icv == JNI_TRUE) (*env)->ReleaseByteArrayElements(env, val, vbuf, JNI_ABORT);
   return rv;
@@ -165,7 +172,7 @@ Java_qdbm_Villa_vlout(JNIEnv *env, jclass myclass, jint index, jbyteArray key, j
   vljnienv = env;
   vlmyclass = myclass;
   kbuf = (*env)->GetByteArrayElements(env, key, &ick);
-  rv = vlout(vltable[index], kbuf, ksiz);
+  rv = vlout(vltable[index], (char *)kbuf, ksiz);
   if(ick == JNI_TRUE) (*env)->ReleaseByteArrayElements(env, key, kbuf, JNI_ABORT);
   return rv;
 }
@@ -182,11 +189,11 @@ Java_qdbm_Villa_vlget(JNIEnv *env, jclass myclass, jint index, jbyteArray key, j
   vljnienv = env;
   vlmyclass = myclass;
   kbuf = (*env)->GetByteArrayElements(env, key, &ick);
-  val = vlgetcache(vltable[index], kbuf, ksiz, &vsiz);
+  val = vlgetcache(vltable[index], (char *)kbuf, ksiz, &vsiz);
   if(ick == JNI_TRUE) (*env)->ReleaseByteArrayElements(env, key, kbuf, JNI_ABORT);
   if(val){
     vbuf = (*env)->NewByteArray(env, vsiz);
-    (*env)->SetByteArrayRegion(env, vbuf, 0, vsiz, val);
+    (*env)->SetByteArrayRegion(env, vbuf, 0, vsiz, (jbyte *)val);
   } else {
     vbuf = NULL;
   }
@@ -203,7 +210,7 @@ Java_qdbm_Villa_vlvsiz(JNIEnv *env, jclass myclass, jint index, jbyteArray key,
   vljnienv = env;
   vlmyclass = myclass;
   kbuf = (*env)->GetByteArrayElements(env, key, &ick);
-  rv = vlvsiz(vltable[index], kbuf, ksiz);
+  rv = vlvsiz(vltable[index], (char *)kbuf, ksiz);
   if(ick == JNI_TRUE) (*env)->ReleaseByteArrayElements(env, key, kbuf, JNI_ABORT);
   return rv;
 }
@@ -218,7 +225,7 @@ Java_qdbm_Villa_vlvnum(JNIEnv *env, jclass myclass, jint index, jbyteArray key,
   vljnienv = env;
   vlmyclass = myclass;
   kbuf = (*env)->GetByteArrayElements(env, key, &ick);
-  rv = vlvnum(vltable[index], kbuf, ksiz);
+  rv = vlvnum(vltable[index], (char *)kbuf, ksiz);
   if(ick == JNI_TRUE) (*env)->ReleaseByteArrayElements(env, key, kbuf, JNI_ABORT);
   return rv;
 }
@@ -270,7 +277,7 @@ Java_qdbm_Villa_vlcurjump(JNIEnv *env, jclass myclass, jint index,
   vljnienv = env;
   vlmyclass = myclass;
   kbuf = (*env)->GetByteArrayElements(env, key, &ick);
-  rv = vlcurjump(vltable[index], kbuf, ksiz, getvljmode(jmode));
+  rv = vlcurjump(vltable[index], (char *)kbuf, ksiz, getvljmode(jmode));
   if(ick == JNI_TRUE) (*env)->ReleaseByteArrayElements(env, key, kbuf, JNI_ABORT);
   return rv;
 }
@@ -287,7 +294,7 @@ Java_qdbm_Villa_vlcurkey(JNIEnv *env, jclass myclass, jint index)
   key = vlcurkeycache(vltable[index], &ksiz);
   if(key){
     kbuf = (*env)->NewByteArray(env, ksiz);
-    (*env)->SetByteArrayRegion(env, kbuf, 0, ksiz, key);
+    (*env)->SetByteArrayRegion(env, kbuf, 0, ksiz, (jbyte *)key);
   } else {
     kbuf = NULL;
   }
@@ -306,7 +313,7 @@ Java_qdbm_Villa_vlcurval(JNIEnv *env, jclass myclass, jint index)
   val = vlcurvalcache(vltable[index], &vsiz);
   if(val){
     vbuf = (*env)->NewByteArray(env, vsiz);
-    (*env)->SetByteArrayRegion(env, vbuf, 0, vsiz, val);
+    (*env)->SetByteArrayRegion(env, vbuf, 0, vsiz, (jbyte *)val);
   } else {
     vbuf = NULL;
   }
@@ -323,7 +330,7 @@ Java_qdbm_Villa_vlcurput(JNIEnv *env, jclass myclass, jint index,
   vljnienv = env;
   vlmyclass = myclass;
   vbuf = (*env)->GetByteArrayElements(env, val, &icv);
-  rv = vlcurput(vltable[index], vbuf, vsiz, getvlcpmode(cpmode));
+  rv = vlcurput(vltable[index], (char *)vbuf, vsiz, getvlcpmode(cpmode));
   if(icv == JNI_TRUE) (*env)->ReleaseByteArrayElements(env, val, vbuf, JNI_ABORT);
   return rv;
 }
@@ -490,7 +497,163 @@ Java_qdbm_Villa_vlremove(JNIEnv *env, jclass myclass, jstring name)
   tname = (*env)->GetStringUTFChars(env, name, &ic);
   rv = vlremove(tname);
   if(ic == JNI_TRUE) (*env)->ReleaseStringUTFChars(env, name, tname);
-  return rv ? 1 : 0;
+  return rv;
+}
+
+
+JNIEXPORT jint JNICALL
+Java_qdbm_VillaCursor_vlmulcurecode(JNIEnv *env, jclass myclass)
+{
+  return Java_qdbm_Villa_vlecode(env, myclass);
+}
+
+
+JNIEXPORT jint
+JNICALL Java_qdbm_VillaCursor_vlmulcurnew(JNIEnv *env, jobject obj, jint index)
+{
+  jclass cls;
+  jfieldID fid;
+  VLMULCUR *mulcur;
+  if(!(mulcur = vlmulcuropen(vltable[index]))) return 0;
+  cls = (*env)->GetObjectClass(env, obj);
+  fid = (*env)->GetFieldID(env, cls, "coreptr", "J");
+  (*env)->SetLongField(env, obj, fid, (PTRNUM)mulcur);
+  return 1;
+}
+
+
+JNIEXPORT void JNICALL
+Java_qdbm_VillaCursor_vlmulcurdelete(JNIEnv *env, jobject obj)
+{
+  jclass cls;
+  jfieldID fid;
+  jlong coreptr;
+  cls = (*env)->GetObjectClass(env, obj);
+  fid = (*env)->GetFieldID(env, cls, "coreptr", "J");
+  coreptr = (*env)->GetLongField(env, obj, fid);
+
+
+  printf("hoge\n");
+
+  vlmulcurclose((VLMULCUR *)(PTRNUM)coreptr);
+}
+
+
+JNIEXPORT jint JNICALL
+Java_qdbm_VillaCursor_vlmulcurfirst(JNIEnv *env, jobject obj, jint index)
+{
+  jclass cls;
+  jfieldID fid;
+  jlong coreptr;
+  cls = (*env)->GetObjectClass(env, obj);
+  fid = (*env)->GetFieldID(env, cls, "coreptr", "J");
+  coreptr = (*env)->GetLongField(env, obj, fid);
+  return vlmulcurfirst((VLMULCUR *)(PTRNUM)coreptr);
+}
+
+
+JNIEXPORT jint JNICALL
+Java_qdbm_VillaCursor_vlmulcurlast(JNIEnv *env, jobject obj, jint index)
+{
+  jclass cls;
+  jfieldID fid;
+  jlong coreptr;
+  cls = (*env)->GetObjectClass(env, obj);
+  fid = (*env)->GetFieldID(env, cls, "coreptr", "J");
+  coreptr = (*env)->GetLongField(env, obj, fid);
+  return vlmulcurlast((VLMULCUR *)(PTRNUM)coreptr);
+}
+
+
+JNIEXPORT jint JNICALL
+Java_qdbm_VillaCursor_vlmulcurprev(JNIEnv *env, jobject obj, jint index)
+{
+  jclass cls;
+  jfieldID fid;
+  jlong coreptr;
+  cls = (*env)->GetObjectClass(env, obj);
+  fid = (*env)->GetFieldID(env, cls, "coreptr", "J");
+  coreptr = (*env)->GetLongField(env, obj, fid);
+  return vlmulcurprev((VLMULCUR *)(PTRNUM)coreptr);
+}
+
+
+JNIEXPORT jint JNICALL
+Java_qdbm_VillaCursor_vlmulcurnext(JNIEnv *env, jobject obj, jint index)
+{
+  jclass cls;
+  jfieldID fid;
+  jlong coreptr;
+  cls = (*env)->GetObjectClass(env, obj);
+  fid = (*env)->GetFieldID(env, cls, "coreptr", "J");
+  coreptr = (*env)->GetLongField(env, obj, fid);
+  return vlmulcurnext((VLMULCUR *)(PTRNUM)coreptr);
+}
+
+
+JNIEXPORT jint JNICALL
+Java_qdbm_VillaCursor_vlmulcurjump(JNIEnv *env, jobject obj, jint index,
+                                   jbyteArray key, jint ksiz, jint jmode)
+{
+  jclass cls;
+  jfieldID fid;
+  jlong coreptr;
+  jbyte *kbuf;
+  jboolean ick;
+  int rv;
+  cls = (*env)->GetObjectClass(env, obj);
+  fid = (*env)->GetFieldID(env, cls, "coreptr", "J");
+  coreptr = (*env)->GetLongField(env, obj, fid);
+  kbuf = (*env)->GetByteArrayElements(env, key, &ick);
+  rv = vlmulcurjump((VLMULCUR *)(PTRNUM)coreptr, (char *)kbuf, ksiz, getvljmode(jmode));
+  if(ick == JNI_TRUE) (*env)->ReleaseByteArrayElements(env, key, kbuf, JNI_ABORT);
+  return rv;
+}
+
+
+JNIEXPORT jbyteArray JNICALL
+Java_qdbm_VillaCursor_vlmulcurkey(JNIEnv *env, jobject obj, jint index)
+{
+  jclass cls;
+  jfieldID fid;
+  jlong coreptr;
+  const char *key;
+  int ksiz;
+  jbyteArray kbuf;
+  cls = (*env)->GetObjectClass(env, obj);
+  fid = (*env)->GetFieldID(env, cls, "coreptr", "J");
+  coreptr = (*env)->GetLongField(env, obj, fid);
+  key = vlmulcurkeycache((VLMULCUR *)(PTRNUM)coreptr, &ksiz);
+  if(key){
+    kbuf = (*env)->NewByteArray(env, ksiz);
+    (*env)->SetByteArrayRegion(env, kbuf, 0, ksiz, (jbyte *)key);
+  } else {
+    kbuf = NULL;
+  }
+  return kbuf;
+}
+
+
+JNIEXPORT jbyteArray JNICALL
+Java_qdbm_VillaCursor_vlmulcurval(JNIEnv *env, jobject obj, jint index)
+{
+  jclass cls;
+  jfieldID fid;
+  jlong coreptr;
+  const char *key;
+  int ksiz;
+  jbyteArray kbuf;
+  cls = (*env)->GetObjectClass(env, obj);
+  fid = (*env)->GetFieldID(env, cls, "coreptr", "J");
+  coreptr = (*env)->GetLongField(env, obj, fid);
+  key = vlmulcurvalcache((VLMULCUR *)(PTRNUM)coreptr, &ksiz);
+  if(key){
+    kbuf = (*env)->NewByteArray(env, ksiz);
+    (*env)->SetByteArrayRegion(env, kbuf, 0, ksiz, (jbyte *)key);
+  } else {
+    kbuf = NULL;
+  }
+  return kbuf;
 }
 
 
@@ -585,9 +748,9 @@ static int objcompare(const char *aptr, int asiz, const char *bptr, int bsiz){
   jbyteArray abuf, bbuf;
   mid = (*vljnienv)->GetStaticMethodID(vljnienv, vlmyclass, "objcompare", "([B[B)I");
   abuf = (*vljnienv)->NewByteArray(vljnienv, asiz);
-  (*vljnienv)->SetByteArrayRegion(vljnienv, abuf, 0, asiz, (char *)aptr);
+  (*vljnienv)->SetByteArrayRegion(vljnienv, abuf, 0, asiz, (jbyte *)aptr);
   bbuf = (*vljnienv)->NewByteArray(vljnienv, bsiz);
-  (*vljnienv)->SetByteArrayRegion(vljnienv, bbuf, 0, bsiz, (char *)bptr);
+  (*vljnienv)->SetByteArrayRegion(vljnienv, bbuf, 0, bsiz, (jbyte *)bptr);
   return (*vljnienv)->CallStaticIntMethod(vljnienv, vlmyclass, mid, abuf, bbuf);
 }
 
