@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * System configurations for QDBM
- *                                                      Copyright (C) 2000-2006 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2007 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -58,6 +58,16 @@ extern "C" {
 #define _SYS_HPUX_
 #define _QDBM_SYSNAME  "HP-UX"
 
+#elif defined(__osf)
+
+#define _SYS_TRU64_
+#define _QDBM_SYSNAME  "Tru64"
+
+#elif defined(_AIX)
+
+#define _SYS_AIX_
+#define _QDBM_SYSNAME  "AIX"
+
 #elif defined(__APPLE__) && defined(__MACH__)
 
 #define _SYS_MACOSX_
@@ -217,6 +227,7 @@ extern "C" {
 
 #if defined(_SYS_MSVC_) || defined(_SYS_MINGW_) || defined(_SYS_CYGWIN_)
 
+#undef UNICODE
 #undef open
 
 #define \
@@ -293,7 +304,7 @@ void *_qdbm_settsd(void *ptr, int size, const void *initval);
 
 #if defined(_SYS_MSVC_) || defined(_SYS_MINGW_) || \
   defined(_SYS_FREEBSD_) || defined(_SYS_NETBSD_) || defined(_SYS_OPENBSD_) || \
-  defined(_SYS_RISCOS_) || defined(MYNOMMAP)
+  defined(_SYS_AIX_) || defined(_SYS_RISCOS_) || defined(MYNOMMAP)
 
 #undef PROT_EXEC
 #undef PROT_READ
@@ -309,6 +320,7 @@ void *_qdbm_settsd(void *ptr, int size, const void *initval);
 #undef mmap
 #undef munmap
 #undef msync
+#undef mflush
 
 #define PROT_EXEC      (1 << 0)
 #define PROT_READ      (1 << 1)
@@ -334,10 +346,21 @@ void *_qdbm_settsd(void *ptr, int size, const void *initval);
   msync(start, length, flags) \
   _qdbm_msync(start, length, flags)
 
+#define \
+  mflush(start, length, flags) \
+  _qdbm_msync(start, length, flags)
+
 void *_qdbm_mmap(void *start, size_t length, int prot, int flags, int fd, off_t offset);
 int _qdbm_munmap(void *start, size_t length);
 int _qdbm_msync(const void *start, size_t length, int flags);
 
+#else
+
+#undef mflush
+#define \
+  mflush(start, length, flags) \
+  (0)
+
 #endif
 
 
