@@ -21,10 +21,10 @@ use warnings;
 
 require Tie::Hash;
 require Exporter;
-require DynaLoader;
-our @ISA = qw(Tie::Hash Exporter DynaLoader);
+require XSLoader;
+our @ISA = qw(Tie::Hash Exporter);
 our $VERSION = '1.0';
-bootstrap Curia $VERSION;
+XSLoader::load('Curia', $VERSION);
 
 use constant TRUE => 1;                  # boolean true
 use constant FALSE => 0;                 # boolean false
@@ -53,7 +53,7 @@ our($errmsg) = "no error";               # message of the last error
 
 ##
 # use Curia;
-# Module `Curia' should be included in application codes.
+# Module `Curia' should be loaded in application codes.
 # An instance of the class `Curia' is used as a database handle.
 #
 
@@ -95,16 +95,16 @@ sub new {
     my($omode) = shift;
     my($bnum) = shift;
     my($dnum) = shift;
-    (defined($name) && length($name) > 0 && scalar(@_) == 0) || return undef();
-    (!$handles{$name}) || return undef();
+    (defined($name) && length($name) > 0 && scalar(@_) == 0) || return undef;
+    (!$handles{$name}) || return undef;
     (defined($omode)) || ($omode = OREADER);
     (defined($bnum)) || ($bnum = -1);
     (defined($dnum)) || ($dnum = -1);
     my($curia) = plcropen($name, $omode, $bnum, $dnum);
     $errmsg = plcrerrmsg();
-    ($curia > 0) || return undef();
+    ($curia > 0) || return undef;
     $handles{$name} = $curia;
-    my $self = [$name, TRUE, undef(), undef(), undef(), undef()];
+    my $self = [$name, TRUE, undef, undef, undef, undef];
     bless($self, $class);
     return $self;
 }
@@ -207,12 +207,12 @@ sub out {
 #
 sub get {
     my($self) = shift;
-    ($$self[1]) || return undef();
+    ($$self[1]) || return undef;
     my($key) = shift;
     my($start) = shift;
     my($max) = shift;
-    (scalar(@_) == 0) || return undef();
-    (defined($key)) || return undef();
+    (scalar(@_) == 0) || return undef;
+    (defined($key)) || return undef;
     (defined($start) && $start >= 0) || ($start = 0);
     (defined($max) && $start >= 0) || ($max = -1);
     my($curia) = $handles{$$self[0]};
@@ -287,8 +287,8 @@ sub iterinit {
 #
 sub iternext {
     my($self) = shift;
-    ($$self[1]) || return undef();
-    (scalar(@_) == 0) || return undef();
+    ($$self[1]) || return undef;
+    (scalar(@_) == 0) || return undef;
     my($curia) = $handles{$$self[0]};
     my($rv) = plcriternext($curia);
     $errmsg = plcrerrmsg();
@@ -487,7 +487,7 @@ sub DESTROY {
 #
 sub TIEHASH {
     my($class, $name, $omode, $bnum, $dnum) = @_;
-    (defined($name)) || return undef();
+    (defined($name)) || return undef;
     (defined($omode)) || ($omode = OWRITER | OCREAT);
     (defined($bnum)) || ($bnum = -1);
     (defined($dnum)) || ($dnum = -1);
@@ -561,8 +561,8 @@ sub CLEAR {
 #
 sub FETCH {
     my($self, $key) = @_;
-    ($$self[1]) || return undef();
-    (defined($key)) || return undef();
+    ($$self[1]) || return undef;
+    (defined($key)) || return undef;
     my($curia) = $handles{$$self[0]};
     if($$self[2]){
         local($_) = $key;
@@ -597,7 +597,7 @@ sub EXISTS {
 #
 sub FIRSTKEY {
     my($self) = shift;
-    ($self->iterinit()) || return undef();
+    ($self->iterinit()) || return undef;
     return $self->iternext();
 }
 
