@@ -21,10 +21,10 @@ use warnings;
 
 require Tie::Hash;
 require Exporter;
-require DynaLoader;
-our @ISA = qw(Tie::Hash Exporter DynaLoader);
+require XSLoader;
+our @ISA = qw(Tie::Hash Exporter);
 our $VERSION = '1.0';
-bootstrap Depot $VERSION;
+XSLoader::load('Depot', $VERSION);
 
 use constant TRUE => 1;                  # boolean true
 use constant FALSE => 0;                 # boolean false
@@ -53,7 +53,7 @@ our($errmsg) = "no error";               # message of the last error
 
 ##
 # use Depot;
-# Module `Depot' should be included in application codes.
+# Module `Depot' should be loaded in application codes.
 # An instance of the class `Depot' is used as a database handle.
 #
 
@@ -91,15 +91,15 @@ sub new {
     my($name) = shift;
     my($omode) = shift;
     my($bnum) = shift;
-    (defined($name) && length($name) > 0 && scalar(@_) == 0) || return undef();
-    (!$handles{$name}) || return undef();
+    (defined($name) && length($name) > 0 && scalar(@_) == 0) || return undef;
+    (!$handles{$name}) || return undef;
     (defined($omode)) || ($omode = OREADER);
     (defined($bnum)) || ($bnum = -1);
     my($depot) = pldpopen($name, $omode, $bnum);
     $errmsg = pldperrmsg();
-    ($depot > 0) || return undef();
+    ($depot > 0) || return undef;
     $handles{$name} = $depot;
-    my $self = [$name, TRUE, undef(), undef(), undef(), undef()];
+    my $self = [$name, TRUE, undef, undef, undef, undef];
     bless($self, $class);
     return $self;
 }
@@ -202,12 +202,12 @@ sub out {
 #
 sub get {
     my($self) = shift;
-    ($$self[1]) || return undef();
+    ($$self[1]) || return undef;
     my($key) = shift;
     my($start) = shift;
     my($max) = shift;
-    (scalar(@_) == 0) || return undef();
-    (defined($key)) || return undef();
+    (scalar(@_) == 0) || return undef;
+    (defined($key)) || return undef;
     (defined($start) && $start >= 0) || ($start = 0);
     (defined($max) && $start >= 0) || ($max = -1);
     my($depot) = $handles{$$self[0]};
@@ -282,8 +282,8 @@ sub iterinit {
 #
 sub iternext {
     my($self) = shift;
-    ($$self[1]) || return undef();
-    (scalar(@_) == 0) || return undef();
+    ($$self[1]) || return undef;
+    (scalar(@_) == 0) || return undef;
     my($depot) = $handles{$$self[0]};
     my($rv) = pldpiternext($depot);
     $errmsg = pldperrmsg();
@@ -481,7 +481,7 @@ sub DESTROY {
 #
 sub TIEHASH {
     my($class, $name, $omode, $bnum) = @_;
-    (defined($name)) || return undef();
+    (defined($name)) || return undef;
     (defined($omode)) || ($omode = OWRITER | OCREAT);
     (defined($bnum)) || ($bnum = -1);
     return $class->new($name, $omode, $bnum);
@@ -554,8 +554,8 @@ sub CLEAR {
 #
 sub FETCH {
     my($self, $key) = @_;
-    ($$self[1]) || return undef();
-    (defined($key)) || return undef();
+    ($$self[1]) || return undef;
+    (defined($key)) || return undef;
     my($depot) = $handles{$$self[0]};
     if($$self[2]){
         local($_) = $key;
@@ -590,7 +590,7 @@ sub EXISTS {
 #
 sub FIRSTKEY {
     my($self) = shift;
-    ($self->iterinit()) || return undef();
+    ($self->iterinit()) || return undef;
     return $self->iternext();
 }
 
