@@ -208,7 +208,7 @@ Datum::Datum(char* dptr, int dsize, bool dummy){
 }
 
 
-Datum operator +(const Datum& former, const Datum& latter){
+Datum qdbm::operator +(const Datum& former, const Datum& latter){
   int len = former.dsize + latter.dsize;
   char *buf = xmalloc(former.dsize + latter.dsize + 1);
   std::memcpy(buf, former.dptr, former.dsize);
@@ -218,7 +218,7 @@ Datum operator +(const Datum& former, const Datum& latter){
 }
 
 
-Datum operator +(const Datum& datum, const char* str){
+Datum qdbm::operator +(const Datum& datum, const char* str){
   int len = strlen(str);
   char* buf = xmalloc(datum.dsize + len + 1);
   std::memcpy(buf, datum.dptr, datum.dsize);
@@ -229,7 +229,7 @@ Datum operator +(const Datum& datum, const char* str){
 }
 
 
-Datum operator +(const char* str, const Datum& datum){
+Datum qdbm::operator +(const char* str, const Datum& datum){
   int len = strlen(str);
   char* buf = xmalloc(len + datum.dsize + 1);
   std::memcpy(buf, str, len);
