@@ -507,7 +507,7 @@ void* mtwriter(void* arg){
 int domisc(const char* name){
   bool err;
   int i, len;
-  char buf[RECBUFSIZ];
+  char buf[RECBUFSIZ], *tmp;
   cout << "<Miscellaneous Test>" << endl;
   cout << "  name=" << name << endl;
   cout << endl;
@@ -559,9 +559,6 @@ int domisc(const char* name){
   }
   if(adbm) delete adbm;
   if(err) return 1;
-
-
-
   Curia* curia = 0;
   try {
     // open the database
@@ -606,6 +603,7 @@ int domisc(const char* name){
     // open the database
     cout << "Opening the database ... ";
     curia = new Curia(name, Curia::OWRITER);
+    curia->silent = true;
     cout << "ok" << endl;
     // sync the database
     cout << "Syncing the database ... ";
@@ -615,6 +613,27 @@ int domisc(const char* name){
     cout << "Optimizing the database ... ";
     curia->optimize();
     cout << "ok" << endl;
+    // write in silent mode
+    cout << "Writing in silent mode ... ";
+    for(i = 1; i <= 100; i++){
+      len = sprintf(buf, "%d", rand() % 100);
+      curia->put(buf, len, buf, len, Curia::DKEEP);
+    }
+    cout << "ok" << endl;
+    // read in silent mode
+    cout << "Reading in silent mode ... ";
+    for(i = 1; i <= 100; i++){
+      len = sprintf(buf, "%d", rand() % 100);
+      if((tmp = curia->get(buf, len)) != 0) free(tmp);
+    }
+    cout << "ok" << endl;
+    // traversal access
+    cout << "Traversal access in silent mode ... ";
+    curia->iterinit();
+    while((tmp = curia->iternext()) != 0){
+      free(tmp);
+    }
+    cout << "ok" << endl;
     // close the database
     cout << "Closing the database ... ";
     curia->close();
