@@ -154,8 +154,9 @@ const char* Curia::version() throw(){
 void Curia::remove(const char* name) throw(Curia_error){
   if(pthread_mutex_lock(&ourmutex) != 0) throw Curia_error();
   if(!crremove(name)){
+    int ecode = dpecode;
     pthread_mutex_unlock(&ourmutex);
-    throw Curia_error(dpecode);
+    throw Curia_error(ecode);
   }
   pthread_mutex_unlock(&ourmutex);
 }
@@ -165,8 +166,9 @@ char* Curia::snaffle(const char* name, const char* kbuf, int ksiz, int* sp) thro
   char* vbuf;
   if(pthread_mutex_lock(&ourmutex) != 0) throw Curia_error();
   if(!(vbuf = crsnaffle(name, kbuf, ksiz, sp))){
+    int ecode = dpecode;
     pthread_mutex_unlock(&ourmutex);
-    throw Curia_error(dpecode);
+    throw Curia_error(ecode);
   }
   pthread_mutex_unlock(&ourmutex);
   return vbuf;
@@ -176,10 +178,12 @@ char* Curia::snaffle(const char* name, const char* kbuf, int ksiz, int* sp) thro
 Curia::Curia(const char* name, int omode, int bnum, int dnum) throw(Curia_error)
   : ADBM(){
   pthread_mutex_init(&mymutex, NULL);
+  silent = false;
   if(!lock()) throw Curia_error();
   if(!(curia = cropen(name, omode, bnum, dnum))){
+    int ecode = dpecode;
     unlock();
-    throw Curia_error(dpecode);
+    throw Curia_error(ecode);
   }
   unlock();
 }
@@ -203,35 +207,42 @@ void Curia::close() throw(Curia_error){
   if(!curia) throw Curia_error();
   if(!lock()) throw Curia_error();
   if(!crclose(curia)){
+    int ecode = dpecode;
     curia = 0;
     unlock();
-    throw Curia_error(dpecode);
+    throw Curia_error(ecode);
   }
   curia = 0;
   unlock();
 }
 
 
-void Curia::put(const char* kbuf, int ksiz, const char* vbuf, int vsiz, int dmode)
+bool Curia::put(const char* kbuf, int ksiz, const char* vbuf, int vsiz, int dmode)
   throw(Curia_error){
   if(!curia) throw Curia_error();
   if(!lock()) throw Curia_error();
   if(!crput(curia, kbuf, ksiz, vbuf, vsiz, dmode)){
+    int ecode = dpecode;
     unlock();
-    throw Curia_error(dpecode);
+    if(silent && ecode == EKEEP) return false;
+    throw Curia_error(ecode);
   }
   unlock();
+  return true;
 }
 
 
-void Curia::out(const char* kbuf, int ksiz) throw(Curia_error){
+bool Curia::out(const char* kbuf, int ksiz) throw(Curia_error){
   if(!curia) throw Curia_error();
   if(!lock()) throw Curia_error();
   if(!crout(curia, kbuf, ksiz)){
+    int ecode = dpecode;
     unlock();
-    throw Curia_error(dpecode);
+    if(silent && ecode == ENOITEM) return false;
+    throw Curia_error(ecode);
   }
   unlock();
+  return true;
 }
 
 
@@ -240,8 +251,10 @@ char* Curia::get(const char* kbuf, int ksiz, int start, int max, int* sp) throw(
   if(!curia) throw Curia_error();
   if(!lock()) throw Curia_error();
   if(!(vbuf = crget(curia, kbuf, ksiz, start, max, sp))){
+    int ecode = dpecode;
     unlock();
-    throw Curia_error(dpecode);
+    if(silent && ecode == ENOITEM) return 0;
+    throw Curia_error(ecode);
   }
   unlock();
   return vbuf;
@@ -253,8 +266,10 @@ int Curia::getwb(const char *kbuf, int ksiz, int start, int max, char *vbuf) thr
   if(!curia) throw Curia_error();
   if(!lock()) throw Curia_error();
   if((vsiz = crgetwb(curia, kbuf, ksiz, start, max, vbuf)) == -1){
+    int ecode = dpecode;
     unlock();
-    throw Curia_error(dpecode);
+    if(silent && ecode == ENOITEM) return -1;
+    throw Curia_error(ecode);
   }
   unlock();
   return vsiz;
@@ -266,8 +281,10 @@ int Curia::vsiz(const char* kbuf, int ksiz) throw(Curia_error){
   if(!curia) throw Curia_error();
   if(!lock()) throw Curia_error();
   if((vsiz = crvsiz(curia, kbuf, ksiz)) == -1){
+    int ecode = dpecode;
     unlock();
-    throw Curia_error(dpecode);
+    if(silent && ecode == ENOITEM) return -1;
+    throw Curia_error(ecode);
   }
   unlock();
   return vsiz;
@@ -278,8 +295,9 @@ void Curia::iterinit() throw(Curia_error){
   if(!curia) throw Curia_error();
   if(!lock()) throw Curia_error();
   if(!criterinit(curia)){
+    int ecode = dpecode;
     unlock();
-    throw Curia_error(dpecode);
+    throw Curia_error(ecode);
   }
   unlock();
 }
@@ -290,8 +308,10 @@ char* Curia::iternext(int* sp) throw(Curia_error){
   if(!curia) throw Curia_error();
   if(!lock()) throw Curia_error();
   if(!(vbuf = criternext(curia, sp))){
+    int ecode = dpecode;
     unlock();
-    throw Curia_error(dpecode);
+    if(silent && ecode == ENOITEM) return 0;
+    throw Curia_error(ecode);
   }
   unlock();
   return vbuf;
@@ -302,8 +322,9 @@ void Curia::setalign(int align) throw(Curia_error){
   if(!curia) throw Curia_error();
   if(!lock()) throw Curia_error();
   if(!crsetalign(curia, align)){
+    int ecode = dpecode;
     unlock();
-    throw Curia_error(dpecode);
+    throw Curia_error(ecode);
   }
   unlock();
 }
@@ -313,8 +334,9 @@ void Curia::setfbpsiz(int size) throw(Curia_error){
   if(!curia) throw Curia_error();
   if(!lock()) throw Curia_error();
   if(!crsetfbpsiz(curia, size)){
+    int ecode = dpecode;
     unlock();
-    throw Curia_error(dpecode);
+    throw Curia_error(ecode);
   }
   unlock();
 }
@@ -324,8 +346,9 @@ void Curia::sync() throw(Curia_error){
   if(!curia) throw Curia_error();
   if(!lock()) throw Curia_error();
   if(!crsync(curia)){
+    int ecode = dpecode;
     unlock();
-    throw Curia_error(dpecode);
+    throw Curia_error(ecode);
   }
   unlock();
 }
@@ -335,8 +358,9 @@ void Curia::optimize(int bnum) throw(Curia_error){
   if(!curia) throw Curia_error();
   if(!lock()) throw Curia_error();
   if(!croptimize(curia, bnum)){
+    int ecode = dpecode;
     unlock();
-    throw Curia_error(dpecode);
+    throw Curia_error(ecode);
   }
   unlock();
 }
@@ -347,8 +371,9 @@ char* Curia::name() throw(Curia_error){
   if(!curia) throw Curia_error();
   if(!lock()) throw Curia_error();
   if(!(buf = crname(curia))){
+    int ecode = dpecode;
     unlock();
-    throw Curia_error(dpecode);
+    throw Curia_error(ecode);
   }
   unlock();
   return buf;
@@ -360,8 +385,9 @@ int Curia::fsiz() throw(Curia_error){
   if(!curia) throw Curia_error();
   if(!lock()) throw Curia_error();
   if((rv = crfsiz(curia)) == -1){
+    int ecode = dpecode;
     unlock();
-    throw Curia_error(dpecode);
+    throw Curia_error(ecode);
   }
   unlock();
   return rv;
@@ -373,8 +399,9 @@ double Curia::fsizd() throw(Curia_error){
   if(!curia) throw Curia_error();
   if(!lock()) throw Curia_error();
   if((rv = crfsizd(curia)) == -1){
+    int ecode = dpecode;
     unlock();
-    throw Curia_error(dpecode);
+    throw Curia_error(ecode);
   }
   unlock();
   return rv;
@@ -386,8 +413,9 @@ int Curia::bnum() throw(Curia_error){
   if(!curia) throw Curia_error();
   if(!lock()) throw Curia_error();
   if((rv = crbnum(curia)) == -1){
+    int ecode = dpecode;
     unlock();
-    throw Curia_error(dpecode);
+    throw Curia_error(ecode);
   }
   unlock();
   return rv;
@@ -399,8 +427,9 @@ int Curia::busenum() throw(Curia_error){
   if(!curia) throw Curia_error();
   if(!lock()) throw Curia_error();
   if((rv = crbusenum(curia)) == -1){
+    int ecode = dpecode;
     unlock();
-    throw Curia_error(dpecode);
+    throw Curia_error(ecode);
   }
   unlock();
   return rv;
@@ -412,8 +441,9 @@ int Curia::rnum() throw(Curia_error){
   if(!curia) throw Curia_error();
   if(!lock()) throw Curia_error();
   if((rv = crrnum(curia)) == -1){
+    int ecode = dpecode;
     unlock();
-    throw Curia_error(dpecode);
+    throw Curia_error(ecode);
   }
   unlock();
   return rv;
@@ -460,26 +490,32 @@ time_t Curia::mtime() throw(Curia_error){
 }
 
 
-void Curia::putlob(const char* kbuf, int ksiz, const char* vbuf, int vsiz, int dmode)
+bool Curia::putlob(const char* kbuf, int ksiz, const char* vbuf, int vsiz, int dmode)
   throw(Curia_error){
   if(!curia) throw Curia_error();
   if(!lock()) throw Curia_error();
   if(!crputlob(curia, kbuf, ksiz, vbuf, vsiz, dmode)){
+    int ecode = dpecode;
     unlock();
-    throw Curia_error(dpecode);
+    if(silent && ecode == EKEEP) return false;
+    throw Curia_error(ecode);
   }
   unlock();
+  return true;
 }
 
 
-void Curia::outlob(const char* kbuf, int ksiz) throw(Curia_error){
+bool Curia::outlob(const char* kbuf, int ksiz) throw(Curia_error){
   if(!curia) throw Curia_error();
   if(!lock()) throw Curia_error();
   if(!croutlob(curia, kbuf, ksiz)){
+    int ecode = dpecode;
     unlock();
-    throw Curia_error(dpecode);
+    if(silent && ecode == ENOITEM) return false;
+    throw Curia_error(ecode);
   }
   unlock();
+  return true;
 }
 
 
@@ -488,8 +524,10 @@ char* Curia::getlob(const char* kbuf, int ksiz, int start, int max, int* sp) thr
   if(!curia) throw Curia_error();
   if(!lock()) throw Curia_error();
   if(!(vbuf = crgetlob(curia, kbuf, ksiz, start, max, sp))){
+    int ecode = dpecode;
     unlock();
-    throw Curia_error(dpecode);
+    if(silent && ecode == ENOITEM) return 0;
+    throw Curia_error(ecode);
   }
   unlock();
   return vbuf;
@@ -501,8 +539,10 @@ int Curia::getlobfd(const char *kbuf, int ksiz){
   if(!curia) throw Curia_error();
   if(!lock()) throw Curia_error();
   if((fd = crgetlobfd(curia, kbuf, ksiz)) == -1){
+    int ecode = dpecode;
     unlock();
-    throw Curia_error(dpecode);
+    if(silent && ecode == ENOITEM) return -1;
+    throw Curia_error(ecode);
   }
   unlock();
   return fd;
@@ -514,8 +554,10 @@ int Curia::vsizlob(const char* kbuf, int ksiz) throw(Curia_error){
   if(!curia) throw Curia_error();
   if(!lock()) throw Curia_error();
   if((vsiz = crvsizlob(curia, kbuf, ksiz)) == -1){
+    int ecode = dpecode;
     unlock();
-    throw Curia_error(dpecode);
+    if(silent && ecode == ENOITEM) return -1;
+    throw Curia_error(ecode);
   }
   unlock();
   return vsiz;
@@ -533,12 +575,13 @@ int Curia::rnumlob() throw(Curia_error){
 
 
 void Curia::storerec(const Datum& key, const Datum& val, bool replace) throw(Curia_error){
-  put(key.ptr(), key.size(), val.ptr(), val.size(), replace ? CR_DOVER : CR_DKEEP);
+  if(!put(key.ptr(), key.size(), val.ptr(), val.size(), replace ? CR_DOVER : CR_DKEEP))
+    throw Curia_error(EKEEP);
 }
 
 
 void Curia::deleterec(const Datum& key) throw(Curia_error){
-  out(key.ptr(), key.size());
+  if(!out(key.ptr(), key.size())) throw Curia_error(ENOITEM);
 }
 
 
@@ -546,6 +589,7 @@ Datum Curia::fetchrec(const Datum& key) throw(Curia_error){
   char* vbuf;
   int vsiz;
   vbuf = get(key.ptr(), key.size(), 0, -1, &vsiz);
+  if(!vbuf) throw Curia_error(ENOITEM);
   return Datum(vbuf, vsiz, true);
 }
 
@@ -560,6 +604,7 @@ Datum Curia::nextkey() throw(Curia_error){
   char* kbuf;
   int ksiz;
   kbuf = iternext(&ksiz);
+  if(!kbuf) throw Curia_error(ENOITEM);
   return Datum(kbuf, ksiz, true);
 }
 
