@@ -242,18 +242,22 @@ public:
    * which means the specified value overwrites the existing one, `Curia::DKEEP', which means the
    * existing value is kept, `Curia::DCAT', which means the specified value is concatenated at
    * the end of the existing value.
+   * @return always true.  However, if the silent flag is true and replace is cancelled, false is
+   * returned instead of exception.
    * @throw Curia_error if an error occurs or replace is cancelled.
    */
-  virtual void put(const char* kbuf, int ksiz, const char* vbuf, int vsiz,
+  virtual bool put(const char* kbuf, int ksiz, const char* vbuf, int vsiz,
                    int dmode = Curia::DOVER) throw(Curia_error);
   /**
    * Delete a record.
    * @param kbuf the pointer to the region of a key.
    * @param ksiz the size of the region of the key.  If it is negative, the size is assigned
    * with `std::strlen(kbuf)'.
+   * @return always true.  However, if the silent flag is true and no record corresponds, false
+   * is returned instead of exception.
    * @throw Curia_error if an error occurs or no record corresponds.
    */
-  virtual void out(const char* kbuf, int ksiz) throw(Curia_error);
+  virtual bool out(const char* kbuf, int ksiz) throw(Curia_error);
   /**
    * Retrieve a record.
    * @param kbuf the pointer to the region of a key.
@@ -263,7 +267,8 @@ public:
    * @param max the max size to read with.  If it is negative, the size is unlimited.
    * @param sp the pointer to a variable to which the size of the region of the return value
    * is assigned.  If it is 0, it is not used.
-   * @return the pointer to the region of the value of the corresponding record.
+   * @return the pointer to the region of the value of the corresponding record.  If the silent
+   * flag is true and no record corresponds, 0 is returned instead of exception.
    * @throw Curia_error if an error occurs, no record corresponds, or the size of the value
    * of the corresponding record is less than `start'.
    * @note Because an additional zero code is appended at the end of the region of the return
@@ -283,7 +288,8 @@ public:
    * writing buffer.
    * @param vbuf the pointer to a buffer into which the value of the corresponding record is
    * written.
-   * @return the return value is the size of the written data.
+   * @return the return value is the size of the written data.  If the silent flag is true and
+   * no record corresponds, -1 is returned instead of exception.
    * @throw Curia_error if an error occurs, no record corresponds, or the size of the value
    * of the corresponding record is less than `start'.
    * @note No additional zero code is appended at the end of the region of the writing buffer.
@@ -295,7 +301,8 @@ public:
    * @param kbuf the pointer to the region of a key.
    * @param ksiz the size of the region of the key.  If it is negative, the size is assigned
    * with `std::strlen(kbuf)'.
-   * @return the size of the value of the corresponding record.
+   * @return the size of the value of the corresponding record.  If the silent flag is true and
+   * no record corresponds, -1 is returned instead of exception.
    * @throw Curia_error if an error occurs or no record corresponds.
    * @note Because this function does not read the entity of a record, it is faster than `get'.
    */
@@ -310,7 +317,8 @@ public:
    * Get the next key of the iterator.
    * @param sp the pointer to a variable to which the size of the region of the return value is
    * assigned.  If it is 0, it is not used.
-   * @return the pointer to the region of the next key.
+   * @return the pointer to the region of the next key.  If the silent flag is true and no record
+   * corresponds, 0 is returned instead of exception.
    * @throw Curia_error if an error occurs or no record is to be get out of the iterator.
    * @note Because an additional zero code is appended at the end of the region of the return
    * value, the return value can be treated as a character string.  Because the region of the
@@ -433,18 +441,22 @@ public:
    * which means the specified value overwrites the existing one, `Curia::DKEEP', which means the
    * existing value is kept, `Curia::DCAT', which means the specified value is concatenated at
    * the end of the existing value.
+   * @return always true.  However, if the silent flag is true and replace is cancelled, false is
+   * returned instead of exception.
    * @throw Curia_error if an error occurs or replace is cancelled.
    */
-  virtual void putlob(const char* kbuf, int ksiz, const char* vbuf, int vsiz,
+  virtual bool putlob(const char* kbuf, int ksiz, const char* vbuf, int vsiz,
                       int dmode = Curia::DOVER) throw(Curia_error);
   /**
    * Delete a large object.
    * @param kbuf the pointer to the region of a key.
    * @param ksiz the size of the region of the key.  If it is negative, the size is assigned
    * with `std::strlen(kbuf)'.
+   * @return always true.  However, if the silent flag is true and no record corresponds, false
+   * is returned instead of exception.
    * @throw Curia_error if an error occurs or no large object corresponds.
    */
-  virtual void outlob(const char* kbuf, int ksiz) throw(Curia_error);
+  virtual bool outlob(const char* kbuf, int ksiz) throw(Curia_error);
   /**
    * Retrieve a large object.
    * @param kbuf the pointer to the region of a key.
@@ -454,7 +466,8 @@ public:
    * @param max the max size to be read.  If it is negative, the size to read is unlimited.
    * @param sp the pointer to a variable to which the size of the region of the return value
    * is assigned.  If it is 0, it is not used.
-   * @return the pointer to the region of the value of the corresponding large object.
+   * @return the pointer to the region of the value of the corresponding large object.  If the
+   * silent flag is true and no record corresponds, 0 is returned instead of exception.
    * @throw Curia_error if an error occurs, no record corresponds, or the size of the value
    * of the corresponding record is less than `start'.
    * @note Because an additional zero code is appended at the end of the region of the return
@@ -469,7 +482,8 @@ public:
    * @param kbuf the pointer to the region of a key.
    * @param ksiz the size of the region of the key.  If it is negative, the size is assigned
    * with `std::strlen(kbuf)'.
-   * @return the file descriptor of the corresponding large object,
+   * @return the file descriptor of the corresponding large object.  If the silent flag is true
+   * and no record corresponds, -1 is returned instead of exception.
    * @throw Curia_error if an error occurs or no record corresponds.
    * @note The returned file descriptor is opened with the `open' call.  If the database handle
    * was opened as a writer, the descriptor is writable (O_RDWR), else, it is not writable
@@ -481,7 +495,8 @@ public:
    * @param kbuf the pointer to the region of a key.
    * @param ksiz the size of the region of the key.  If it is negative, the size is assigned
    * with `std::strlen(kbuf)'.
-   * @return the size of the value of the corresponding large object.
+   * @return the size of the value of the corresponding large object.  If the silent flag is true
+   * and no record corresponds, -1 is returned instead of exception.
    * @throw Curia_error if an error occurs or no large object corresponds.
    * @note Because this function does not read the entity of a large object, it is faster than
    * `getlob'.
@@ -534,6 +549,11 @@ public:
    */
   virtual bool error() throw(Curia_error);
   //----------------------------------------------------------------
+  // public member variables
+  //----------------------------------------------------------------
+public:
+  bool silent;                        ///< whether to repress frequent exceptions
+  //----------------------------------------------------------------
   // private member variables
   //----------------------------------------------------------------
 private:
