@@ -153,8 +153,9 @@ const char* Depot::version() throw(){
 void Depot::remove(const char* name) throw(Depot_error){
   if(pthread_mutex_lock(&ourmutex) != 0) throw Depot_error();
   if(!dpremove(name)){
+    int ecode = dpecode;
     pthread_mutex_unlock(&ourmutex);
-    throw Depot_error(dpecode);
+    throw Depot_error(ecode);
   }
   pthread_mutex_unlock(&ourmutex);
 }
@@ -164,8 +165,9 @@ char* Depot::snaffle(const char* name, const char* kbuf, int ksiz, int* sp) thro
   char* vbuf;
   if(pthread_mutex_lock(&ourmutex) != 0) throw Depot_error();
   if(!(vbuf = dpsnaffle(name, kbuf, ksiz, sp))){
+    int ecode = dpecode;
     pthread_mutex_unlock(&ourmutex);
-    throw Depot_error(dpecode);
+    throw Depot_error(ecode);
   }
   pthread_mutex_unlock(&ourmutex);
   return vbuf;
@@ -175,10 +177,12 @@ char* Depot::snaffle(const char* name, const char* kbuf, int ksiz, int* sp) thro
 Depot::Depot(const char* name, int omode, int bnum) throw(Depot_error)
   : ADBM(){
   pthread_mutex_init(&mymutex, NULL);
+  silent = false;
   if(!lock()) throw Depot_error();
   if(!(depot = dpopen(name, omode, bnum))){
+    int ecode = dpecode;
     unlock();
-    throw Depot_error(dpecode);
+    throw Depot_error(ecode);
   }
   unlock();
 }
@@ -202,35 +206,42 @@ void Depot::close() throw(Depot_error){
   if(!depot) throw Depot_error();
   if(!lock()) throw Depot_error();
   if(!dpclose(depot)){
+    int ecode = dpecode;
     depot = 0;
     unlock();
-    throw Depot_error(dpecode);
+    throw Depot_error(ecode);
   }
   depot = 0;
   unlock();
 }
 
 
-void Depot::put(const char* kbuf, int ksiz, const char* vbuf, int vsiz, int dmode)
+bool Depot::put(const char* kbuf, int ksiz, const char* vbuf, int vsiz, int dmode)
   throw(Depot_error){
   if(!depot) throw Depot_error();
   if(!lock()) throw Depot_error();
   if(!dpput(depot, kbuf, ksiz, vbuf, vsiz, dmode)){
+    int ecode = dpecode;
     unlock();
-    throw Depot_error(dpecode);
+    if(silent && ecode == EKEEP) return false;
+    throw Depot_error(ecode);
   }
   unlock();
+  return true;
 }
 
 
-void Depot::out(const char* kbuf, int ksiz) throw(Depot_error){
+bool Depot::out(const char* kbuf, int ksiz) throw(Depot_error){
   if(!depot) throw Depot_error();
   if(!lock()) throw Depot_error();
   if(!dpout(depot, kbuf, ksiz)){
+    int ecode = dpecode;
     unlock();
-    throw Depot_error(dpecode);
+    if(silent && ecode == ENOITEM) return false;
+    throw Depot_error(ecode);
   }
   unlock();
+  return true;
 }
 
 
@@ -239,8 +250,10 @@ char* Depot::get(const char* kbuf, int ksiz, int start, int max, int* sp) throw(
   if(!depot) throw Depot_error();
   if(!lock()) throw Depot_error();
   if(!(vbuf = dpget(depot, kbuf, ksiz, start, max, sp))){
+    int ecode = dpecode;
     unlock();
-    throw Depot_error(dpecode);
+    if(silent && ecode == ENOITEM) return 0;
+    throw Depot_error(ecode);
   }
   unlock();
   return vbuf;
@@ -252,8 +265,10 @@ int Depot::getwb(const char *kbuf, int ksiz, int start, int max, char *vbuf) thr
   if(!depot) throw Depot_error();
   if(!lock()) throw Depot_error();
   if((vsiz = dpgetwb(depot, kbuf, ksiz, start, max, vbuf)) == -1){
+    int ecode = dpecode;
     unlock();
-    throw Depot_error(dpecode);
+    if(silent && ecode == ENOITEM) return -1;
+    throw Depot_error(ecode);
   }
   unlock();
   return vsiz;
@@ -265,8 +280,10 @@ int Depot::vsiz(const char* kbuf, int ksiz) throw(Depot_error){
   if(!depot) throw Depot_error();
   if(!lock()) throw Depot_error();
   if((vsiz = dpvsiz(depot, kbuf, ksiz)) == -1){
+    int ecode = dpecode;
     unlock();
-    throw Depot_error(dpecode);
+    if(silent && ecode == ENOITEM) return -1;
+    throw Depot_error(ecode);
   }
   unlock();
   return vsiz;
@@ -277,8 +294,9 @@ void Depot::iterinit() throw(Depot_error){
   if(!depot) throw Depot_error();
   if(!lock()) throw Depot_error();
   if(!dpiterinit(depot)){
+    int ecode = dpecode;
     unlock();
-    throw Depot_error(dpecode);
+    throw Depot_error(ecode);
   }
   unlock();
 }
@@ -289,8 +307,10 @@ char* Depot::iternext(int* sp) throw(Depot_error){
   if(!depot) throw Depot_error();
   if(!lock()) throw Depot_error();
   if(!(vbuf = dpiternext(depot, sp))){
+    int ecode = dpecode;
     unlock();
-    throw Depot_error(dpecode);
+    if(silent && ecode == ENOITEM) return 0;
+    throw Depot_error(ecode);
   }
   unlock();
   return vbuf;
@@ -301,8 +321,9 @@ void Depot::setalign(int align) throw(Depot_error){
   if(!depot) throw Depot_error();
   if(!lock()) throw Depot_error();
   if(!dpsetalign(depot, align)){
+    int ecode = dpecode;
     unlock();
-    throw Depot_error(dpecode);
+    throw Depot_error(ecode);
   }
   unlock();
 }
@@ -312,8 +333,9 @@ void Depot::setfbpsiz(int size) throw(Depot_error){
   if(!depot) throw Depot_error();
   if(!lock()) throw Depot_error();
   if(!dpsetfbpsiz(depot, size)){
+    int ecode = dpecode;
     unlock();
-    throw Depot_error(dpecode);
+    throw Depot_error(ecode);
   }
   unlock();
 }
@@ -323,8 +345,9 @@ void Depot::sync() throw(Depot_error){
   if(!depot) throw Depot_error();
   if(!lock()) throw Depot_error();
   if(!dpsync(depot)){
+    int ecode = dpecode;
     unlock();
-    throw Depot_error(dpecode);
+    throw Depot_error(ecode);
   }
   unlock();
 }
@@ -334,8 +357,9 @@ void Depot::optimize(int bnum) throw(Depot_error){
   if(!depot) throw Depot_error();
   if(!lock()) throw Depot_error();
   if(!dpoptimize(depot, bnum)){
+    int ecode = dpecode;
     unlock();
-    throw Depot_error(dpecode);
+    throw Depot_error(ecode);
   }
   unlock();
 }
@@ -346,8 +370,9 @@ char* Depot::name() throw(Depot_error){
   if(!depot) throw Depot_error();
   if(!lock()) throw Depot_error();
   if(!(buf = dpname(depot))){
+    int ecode = dpecode;
     unlock();
-    throw Depot_error(dpecode);
+    throw Depot_error(ecode);
   }
   unlock();
   return buf;
@@ -359,8 +384,9 @@ int Depot::fsiz() throw(Depot_error){
   if(!depot) throw Depot_error();
   if(!lock()) throw Depot_error();
   if((rv = dpfsiz(depot)) == -1){
+    int ecode = dpecode;
     unlock();
-    throw Depot_error(dpecode);
+    throw Depot_error(ecode);
   }
   unlock();
   return rv;
@@ -372,8 +398,9 @@ int Depot::bnum() throw(Depot_error){
   if(!depot) throw Depot_error();
   if(!lock()) throw Depot_error();
   if((rv = dpbnum(depot)) == -1){
+    int ecode = dpecode;
     unlock();
-    throw Depot_error(dpecode);
+    throw Depot_error(ecode);
   }
   unlock();
   return rv;
@@ -385,8 +412,9 @@ int Depot::busenum() throw(Depot_error){
   if(!depot) throw Depot_error();
   if(!lock()) throw Depot_error();
   if((rv = dpbusenum(depot)) == -1){
+    int ecode = dpecode;
     unlock();
-    throw Depot_error(dpecode);
+    throw Depot_error(ecode);
   }
   unlock();
   return rv;
@@ -398,8 +426,9 @@ int Depot::rnum() throw(Depot_error){
   if(!depot) throw Depot_error();
   if(!lock()) throw Depot_error();
   if((rv = dprnum(depot)) == -1){
+    int ecode = dpecode;
     unlock();
-    throw Depot_error(dpecode);
+    throw Depot_error(ecode);
   }
   unlock();
   return rv;
@@ -457,12 +486,13 @@ int Depot::fdesc() throw(Depot_error){
 
 
 void Depot::storerec(const Datum& key, const Datum& val, bool replace) throw(Depot_error){
-  put(key.ptr(), key.size(), val.ptr(), val.size(), replace ? DP_DOVER : DP_DKEEP);
+  if(!put(key.ptr(), key.size(), val.ptr(), val.size(), replace ? DP_DOVER : DP_DKEEP))
+    throw Depot_error(EKEEP);
 }
 
 
 void Depot::deleterec(const Datum& key) throw(Depot_error){
-  out(key.ptr(), key.size());
+  if(!out(key.ptr(), key.size())) throw Depot_error(ENOITEM);
 }
 
 
@@ -470,6 +500,7 @@ Datum Depot::fetchrec(const Datum& key) throw(Depot_error){
   char* vbuf;
   int vsiz;
   vbuf = get(key.ptr(), key.size(), 0, -1, &vsiz);
+  if(!vbuf) throw Depot_error(ENOITEM);
   return Datum(vbuf, vsiz, true);
 }
 
@@ -484,6 +515,7 @@ Datum Depot::nextkey() throw(Depot_error){
   char* kbuf;
   int ksiz;
   kbuf = iternext(&ksiz);
+  if(!kbuf) throw Depot_error(ENOITEM);
   return Datum(kbuf, ksiz, true);
 }
 
