@@ -455,7 +455,7 @@ void* mtwriter(void* arg){
 int domisc(const char* name){
   bool err;
   int i, len;
-  char buf[RECBUFSIZ];
+  char buf[RECBUFSIZ], *tmp;
   cout << "<Miscellaneous Test>" << endl;
   cout << "  name=" << name << endl;
   cout << endl;
@@ -551,6 +551,7 @@ int domisc(const char* name){
     // open the database
     cout << "Opening the database ... ";
     depot = new Depot(name, Depot::OWRITER);
+    depot->silent = true;
     cout << "ok" << endl;
     // sync the database
     cout << "Syncing the database ... ";
@@ -560,6 +561,27 @@ int domisc(const char* name){
     cout << "Optimizing the database ... ";
     depot->optimize();
     cout << "ok" << endl;
+    // write in silent mode
+    cout << "Writing in silent mode ... ";
+    for(i = 1; i <= 100; i++){
+      len = sprintf(buf, "%d", rand() % 100);
+      depot->put(buf, len, buf, len, Depot::DKEEP);
+    }
+    cout << "ok" << endl;
+    // read in silent mode
+    cout << "Reading in silent mode ... ";
+    for(i = 1; i <= 100; i++){
+      len = sprintf(buf, "%d", rand() % 100);
+      if((tmp = depot->get(buf, len)) != 0) free(tmp);
+    }
+    cout << "ok" << endl;
+    // traversal access
+    cout << "Traversal access in silent mode ... ";
+    depot->iterinit();
+    while((tmp = depot->iternext()) != 0){
+      free(tmp);
+    }
+    cout << "ok" << endl;
     // close the database
     cout << "Closing the database ... ";
     depot->close();
