@@ -23,17 +23,17 @@
  *************************************************************************************************/
 
 
-pthread_mutex_t qdbm::mutex;
+pthread_mutex_t qdbm::ourmutex;
 
 
 namespace {
   class Init {
   public:
     Init(){
-      pthread_mutex_init(&qdbm::mutex, NULL);
+      pthread_mutex_init(&qdbm::ourmutex, NULL);
     }
     ~Init(){
-      pthread_mutex_destroy(&qdbm::mutex);
+      pthread_mutex_destroy(&qdbm::ourmutex);
     }
   };
   Init init;
