@@ -39,14 +39,14 @@ namespace qdbm {
   class Curia;
   class Villa_error;
   class Villa;
-  extern pthread_mutex_t mutex;
+  extern pthread_mutex_t ourmutex;
 }
 
 
 /**
  * Common Mutex of QDBM.
  */
-extern pthread_mutex_t qdbm::mutex;
+extern pthread_mutex_t qdbm::ourmutex;
 
 
 
