@@ -164,8 +164,9 @@ const char* Villa::version() throw(){
 void Villa::remove(const char* name) throw(Villa_error){
   if(pthread_mutex_lock(&ourmutex) != 0) throw Villa_error();
   if(!vlremove(name)){
+    int ecode = dpecode;
     pthread_mutex_unlock(&ourmutex);
-    throw Villa_error(dpecode);
+    throw Villa_error(ecode);
   }
   pthread_mutex_unlock(&ourmutex);
 }
@@ -194,10 +195,12 @@ int Villa::cmpdec(const char *aptr, int asiz, const char *bptr, int bsiz) throw(
 Villa::Villa(const char* name, int omode, VLCFUNC cmp) throw(Villa_error)
   : ADBM(){
   pthread_mutex_init(&mymutex, NULL);
+  silent = false;
   if(!lock()) throw Villa_error();
   if(!(villa = vlopen(name, omode, cmp))){
+    int ecode = dpecode;
     unlock();
-    throw Villa_error(dpecode);
+    throw Villa_error(ecode);
   }
   pthread_mutex_init(&tranmutex, NULL);
   unlock();
@@ -223,35 +226,42 @@ void Villa::close() throw(Villa_error){
   if(!villa) throw Villa_error();
   if(!lock()) throw Villa_error();
   if(!vlclose(villa)){
+    int ecode = dpecode;
     villa = 0;
     unlock();
-    throw Villa_error(dpecode);
+    throw Villa_error(ecode);
   }
   villa = 0;
   unlock();
 }
 
 
-void Villa::put(const char* kbuf, int ksiz, const char* vbuf, int vsiz, int dmode)
+bool Villa::put(const char* kbuf, int ksiz, const char* vbuf, int vsiz, int dmode)
   throw(Villa_error){
   if(!villa) throw Villa_error();
   if(!lock()) throw Villa_error();
   if(!vlput(villa, kbuf, ksiz, vbuf, vsiz, dmode)){
+    int ecode = dpecode;
     unlock();
-    throw Villa_error(dpecode);
+    if(silent && ecode == EKEEP) return false;
+    throw Villa_error(ecode);
   }
   unlock();
+  return true;
 }
 
 
-void Villa::out(const char* kbuf, int ksiz) throw(Villa_error){
+bool Villa::out(const char* kbuf, int ksiz) throw(Villa_error){
   if(!villa) throw Villa_error();
   if(!lock()) throw Villa_error();
   if(!vlout(villa, kbuf, ksiz)){
+    int ecode = dpecode;
     unlock();
-    throw Villa_error(dpecode);
+    if(silent && ecode == ENOITEM) return false;
+    throw Villa_error(ecode);
   }
   unlock();
+  return true;
 }
 
 
@@ -260,8 +270,10 @@ char* Villa::get(const char* kbuf, int ksiz, int* sp) throw(Villa_error){
   if(!villa) throw Villa_error();
   if(!lock()) throw Villa_error();
   if(!(vbuf = vlget(villa, kbuf, ksiz, sp))){
+    int ecode = dpecode;
     unlock();
-    throw Villa_error(dpecode);
+    if(silent && ecode == ENOITEM) return 0;
+    throw Villa_error(ecode);
   }
   unlock();
   return vbuf;
@@ -269,12 +281,17 @@ char* Villa::get(const char* kbuf, int ksiz, int* sp) throw(Villa_error){
 
 
 int Villa::vsiz(const char *kbuf, int ksiz) throw(Villa_error){
-  int vnum;
+  int vsiz;
   if(!villa) throw Villa_error();
   if(!lock()) throw Villa_error();
-  vnum = vlvsiz(villa, kbuf, ksiz);
+  if((vsiz = vlvsiz(villa, kbuf, ksiz)) == -1){
+    int ecode = dpecode;
+    unlock();
+    if(silent && ecode == ENOITEM) return 0;
+    throw Villa_error(ecode);
+  }
   unlock();
-  return vnum;
+  return vsiz;
 }
 
 
@@ -288,58 +305,73 @@ int Villa::vnum(const char *kbuf, int ksiz) throw(Villa_error){
 }
 
 
-void Villa::curfirst() throw(Villa_error){
+bool Villa::curfirst() throw(Villa_error){
   if(!villa) throw Villa_error();
   if(!lock()) throw Villa_error();
   if(!vlcurfirst(villa)){
+    int ecode = dpecode;
     unlock();
-    throw Villa_error(dpecode);
+    if(silent && ecode == ENOITEM) return 0;
+    throw Villa_error(ecode);
   }
   unlock();
+  return true;
 }
 
 
-void Villa::curlast() throw(Villa_error){
+bool Villa::curlast() throw(Villa_error){
   if(!villa) throw Villa_error();
   if(!lock()) throw Villa_error();
   if(!vlcurlast(villa)){
+    int ecode = dpecode;
     unlock();
-    throw Villa_error(dpecode);
+    if(silent && ecode == ENOITEM) return 0;
+    throw Villa_error(ecode);
   }
   unlock();
+  return true;
 }
 
 
-void Villa::curprev() throw(Villa_error){
+bool Villa::curprev() throw(Villa_error){
   if(!villa) throw Villa_error();
   if(!lock()) throw Villa_error();
   if(!vlcurprev(villa)){
+    int ecode = dpecode;
     unlock();
-    throw Villa_error(dpecode);
+    if(silent && ecode == ENOITEM) return 0;
+    throw Villa_error(ecode);
   }
   unlock();
+  return true;
 }
 
 
-void Villa::curnext() throw(Villa_error){
+bool Villa::curnext() throw(Villa_error){
   if(!villa) throw Villa_error();
   if(!lock()) throw Villa_error();
   if(!vlcurnext(villa)){
+    int ecode = dpecode;
     unlock();
-    throw Villa_error(dpecode);
+    if(silent && ecode == ENOITEM) return 0;
+    throw Villa_error(ecode);
   }
   unlock();
+  return true;
 }
 
 
-void Villa::curjump(const char* kbuf, int ksiz, int jmode) throw(Villa_error){
+bool Villa::curjump(const char* kbuf, int ksiz, int jmode) throw(Villa_error){
   if(!villa) throw Villa_error();
   if(!lock()) throw Villa_error();
   if(!vlcurjump(villa, kbuf, ksiz, jmode)){
+    int ecode = dpecode;
     unlock();
-    throw Villa_error(dpecode);
+    if(silent && ecode == ENOITEM) return 0;
+    throw Villa_error(ecode);
   }
   unlock();
+  return true;
 }
 
 
@@ -348,8 +380,10 @@ char* Villa::curkey(int *sp) throw(Villa_error){
   if(!villa) throw Villa_error();
   if(!lock()) throw Villa_error();
   if(!(kbuf = vlcurkey(villa, sp))){
+    int ecode = dpecode;
     unlock();
-    throw Villa_error(dpecode);
+    if(silent && ecode == ENOITEM) return 0;
+    throw Villa_error(ecode);
   }
   unlock();
   return kbuf;
@@ -361,33 +395,41 @@ char* Villa::curval(int *sp) throw(Villa_error){
   if(!villa) throw Villa_error();
   if(!lock()) throw Villa_error();
   if(!(vbuf = vlcurval(villa, sp))){
+    int ecode = dpecode;
     unlock();
-    throw Villa_error(dpecode);
+    if(silent && ecode == ENOITEM) return 0;
+    throw Villa_error(ecode);
   }
   unlock();
   return vbuf;
 }
 
 
-void Villa::curput(const char* vbuf, int vsiz, int cpmode) throw(Villa_error){
+bool Villa::curput(const char* vbuf, int vsiz, int cpmode) throw(Villa_error){
   if(!villa) throw Villa_error();
   if(!lock()) throw Villa_error();
   if(!vlcurput(villa, vbuf, vsiz, cpmode)){
+    int ecode = dpecode;
     unlock();
-    throw Villa_error(dpecode);
+    if(silent && ecode == ENOITEM) return false;
+    throw Villa_error(ecode);
   }
   unlock();
+  return true;
 }
 
 
-void Villa::curout() throw(Villa_error){
+bool Villa::curout() throw(Villa_error){
   if(!villa) throw Villa_error();
   if(!lock()) throw Villa_error();
   if(!vlcurout(villa)){
+    int ecode = dpecode;
     unlock();
-    throw Villa_error(dpecode);
+    if(silent && ecode == ENOITEM) return false;
+    throw Villa_error(ecode);
   }
   unlock();
+  return true;
 }
 
 
@@ -403,8 +445,9 @@ void Villa::sync() throw(Villa_error){
   if(!villa) throw Villa_error();
   if(!lock()) throw Villa_error();
   if(!vlsync(villa)){
+    int ecode = dpecode;
     unlock();
-    throw Villa_error(dpecode);
+    throw Villa_error(ecode);
   }
   unlock();
 }
@@ -414,8 +457,9 @@ void Villa::optimize() throw(Villa_error){
   if(!villa) throw Villa_error();
   if(!lock()) throw Villa_error();
   if(!vloptimize(villa)){
+    int ecode = dpecode;
     unlock();
-    throw Villa_error(dpecode);
+    throw Villa_error(ecode);
   }
   unlock();
 }
@@ -426,8 +470,9 @@ char* Villa::name() throw(Villa_error){
   if(!villa) throw Villa_error();
   if(!lock()) throw Villa_error();
   if(!(buf = vlname(villa))){
+    int ecode = dpecode;
     unlock();
-    throw Villa_error(dpecode);
+    throw Villa_error(ecode);
   }
   unlock();
   return buf;
@@ -439,8 +484,9 @@ int Villa::fsiz() throw(Villa_error){
   if(!villa) throw Villa_error();
   if(!lock()) throw Villa_error();
   if((rv = vlfsiz(villa)) == -1){
+    int ecode = dpecode;
     unlock();
-    throw Villa_error(dpecode);
+    throw Villa_error(ecode);
   }
   unlock();
   return rv;
@@ -452,8 +498,9 @@ int Villa::lnum() throw(Villa_error){
   if(!villa) throw Villa_error();
   if(!lock()) throw Villa_error();
   if((rv = vllnum(villa)) == -1){
+    int ecode = dpecode;
     unlock();
-    throw Villa_error(dpecode);
+    throw Villa_error(ecode);
   }
   unlock();
   return rv;
@@ -465,8 +512,9 @@ int Villa::nnum() throw(Villa_error){
   if(!villa) throw Villa_error();
   if(!lock()) throw Villa_error();
   if((rv = vlnnum(villa)) == -1){
+    int ecode = dpecode;
     unlock();
-    throw Villa_error(dpecode);
+    throw Villa_error(ecode);
   }
   unlock();
   return rv;
@@ -478,8 +526,9 @@ int Villa::rnum() throw(Villa_error){
   if(!villa) throw Villa_error();
   if(!lock()) throw Villa_error();
   if((rv = vlrnum(villa)) == -1){
+    int ecode = dpecode;
     unlock();
-    throw Villa_error(dpecode);
+    throw Villa_error(ecode);
   }
   unlock();
   return rv;
@@ -534,9 +583,10 @@ void Villa::tranbegin() throw(Villa_error){
     throw Villa_error();
   }
   if(!vltranbegin(villa)){
+    int ecode = dpecode;
     unlock();
     pthread_mutex_unlock(&tranmutex);
-    throw Villa_error(dpecode);
+    throw Villa_error(ecode);
   }
   unlock();
 }
@@ -546,9 +596,10 @@ void Villa::trancommit() throw(Villa_error){
   if(!villa) throw Villa_error();
   if(!lock()) throw Villa_error();
   if(!vltrancommit(villa)){
+    int ecode = dpecode;
     unlock();
     pthread_mutex_unlock(&tranmutex);
-    throw Villa_error(dpecode);
+    throw Villa_error(ecode);
   }
   unlock();
   if(pthread_mutex_unlock(&tranmutex) != 0) throw Villa_error();
@@ -559,9 +610,10 @@ void Villa::tranabort() throw(Villa_error){
   if(!villa) throw Villa_error();
   if(!lock()) throw Villa_error();
   if(!vltranabort(villa)){
+    int ecode = dpecode;
     unlock();
     pthread_mutex_unlock(&tranmutex);
-    throw Villa_error(dpecode);
+    throw Villa_error(ecode);
   }
   unlock();
   if(pthread_mutex_unlock(&tranmutex) != 0) throw Villa_error();
@@ -569,12 +621,13 @@ void Villa::tranabort() throw(Villa_error){
 
 
 void Villa::storerec(const Datum& key, const Datum& val, bool replace) throw(Villa_error){
-  put(key.ptr(), key.size(), val.ptr(), val.size(), replace ? VL_DOVER : VL_DKEEP);
+  if(!put(key.ptr(), key.size(), val.ptr(), val.size(), replace ? VL_DOVER : VL_DKEEP))
+    throw Villa_error(EKEEP);
 }
 
 
 void Villa::deleterec(const Datum& key) throw(Villa_error){
-  out(key.ptr(), key.size());
+  if(!out(key.ptr(), key.size())) throw Villa_error(ENOITEM);
 }
 
 
@@ -582,6 +635,7 @@ Datum Villa::fetchrec(const Datum& key) throw(Villa_error){
   char* vbuf;
   int vsiz;
   vbuf = get(key.ptr(), key.size(), &vsiz);
+  if(!vbuf) throw Villa_error(ENOITEM);
   return Datum(vbuf, vsiz, true);
 }
 
@@ -591,6 +645,7 @@ Datum Villa::firstkey() throw(Villa_error){
   int ksiz;
   curfirst();
   kbuf = curkey(&ksiz);
+  if(!kbuf) throw Villa_error(ENOITEM);
   return Datum(kbuf, ksiz, true);
 }
 
@@ -600,6 +655,7 @@ Datum Villa::nextkey() throw(Villa_error){
   int ksiz;
   curnext();
   kbuf = curkey(&ksiz);
+  if(!kbuf) throw Villa_error(ENOITEM);
   return Datum(kbuf, ksiz, true);
 }
 
