@@ -49,6 +49,7 @@ VALUE ccuriaerror_ERMDIR;
 VALUE ccuriaerror_EMISC;
 VALUE mcuria;
 CURIA *crtable[MAXOPEN];
+char crsltable[MAXOPEN];
 
 
 static void crinit(void);
@@ -59,6 +60,7 @@ static VALUE rbcrversion(VALUE vself);
 static VALUE rbcrerrmsg(VALUE vself, VALUE vecode);
 static VALUE rbcropen(VALUE vself, VALUE vname, VALUE vomode, VALUE vbnum, VALUE vdnum);
 static VALUE rbcrclose(VALUE vself, VALUE vindex);
+static VALUE rbcrsetsilent(VALUE vself, VALUE vindex, VALUE vvalue);
 static VALUE rbcrput(VALUE vself, VALUE vindex, VALUE vkey, VALUE vval, VALUE vdmode);
 static VALUE rbcrout(VALUE vself, VALUE vindex, VALUE vkey);
 static VALUE rbcrget(VALUE vself, VALUE vindex, VALUE vkey, VALUE vstart, VALUE vmax);
@@ -141,6 +143,7 @@ Init_mod_curia(){
   rb_define_const(mcuria, "DCAT", INT2FIX(CR_DCAT));
   rb_define_module_function(mcuria, "mod_open", rbcropen, 4);
   rb_define_module_function(mcuria, "mod_close", rbcrclose, 1);
+  rb_define_module_function(mcuria, "mod_setsilent", rbcrsetsilent, 2);
   rb_define_module_function(mcuria, "mod_put", rbcrput, 4);
   rb_define_module_function(mcuria, "mod_out", rbcrout, 2);
   rb_define_module_function(mcuria, "mod_get", rbcrget, 4);
@@ -169,6 +172,7 @@ static void crinit(void){
   int i;
   for(i = 0; i < MAXOPEN; i++){
     crtable[i] = NULL;
+    crsltable[i] = 0;
   }
 }
 
@@ -240,6 +244,7 @@ static VALUE rbcropen(VALUE vself, VALUE vname, VALUE vomode, VALUE vbnum, VALUE
   curia = cropen(name, omode, bnum, dnum);
   if(!curia) myerror(dpecode);
   crtable[index] = curia;
+  crsltable[index] = 0;
   return INT2FIX(index);
 }
 
@@ -256,6 +261,15 @@ static VALUE rbcrclose(VALUE vself, VALUE vindex){
 }
 
 
+static VALUE rbcrsetsilent(VALUE vself, VALUE vindex, VALUE vvalue){
+  int index;
+  FIXNUM_P(vindex);
+  if((index = FIX2INT(vindex)) == -1) myerror(DP_EMISC);
+  crsltable[index] = FIX2INT(vvalue);
+  return Qnil;
+}
+
+
 static VALUE rbcrput(VALUE vself, VALUE vindex, VALUE vkey, VALUE vval, VALUE vdmode){
   CURIA *curia;
   const char *kbuf, *vbuf;
@@ -269,7 +283,10 @@ static VALUE rbcrput(VALUE vself, VALUE vindex, VALUE vkey, VALUE vval, VALUE vd
   FIXNUM_P(vdmode);
   dmode = FIX2INT(vdmode);
   curia = crtable[index];
-  if(!crput(curia, kbuf, ksiz, vbuf, vsiz, dmode)) myerror(dpecode);
+  if(!crput(curia, kbuf, ksiz, vbuf, vsiz, dmode)){
+    if(crsltable[index] && dpecode == DP_EKEEP) return Qfalse;
+    myerror(dpecode);
+  }
   return Qtrue;
 }
 
@@ -283,7 +300,10 @@ static VALUE rbcrout(VALUE vself, VALUE vindex, VALUE vkey){
   kbuf = STR2CSTR(vkey);
   ksiz = RSTRING(vkey)->len;
   curia = crtable[index];
-  if(!crout(curia, kbuf, ksiz)) myerror(dpecode);
+  if(!crout(curia, kbuf, ksiz)){
+    if(crsltable[index] && dpecode == DP_ENOITEM) return Qfalse;
+    myerror(dpecode);
+  }
   return Qtrue;
 }
 
@@ -303,7 +323,10 @@ static VALUE rbcrget(VALUE vself, VALUE vindex, VALUE vkey, VALUE vstart, VALUE
   FIXNUM_P(vmax);
   max = FIX2INT(vmax);
   curia = crtable[index];
-  if(!(vbuf = crget(curia, kbuf, ksiz, start, max, &vsiz))) myerror(dpecode);
+  if(!(vbuf = crget(curia, kbuf, ksiz, start, max, &vsiz))){
+    if(crsltable[index] && dpecode == DP_ENOITEM) return Qnil;
+    myerror(dpecode);
+  }
   vval = rb_str_new(vbuf, vsiz);
   free(vbuf);
   return vval;
@@ -319,7 +342,10 @@ static VALUE rbcrvsiz(VALUE vself, VALUE vindex, VALUE vkey){
   kbuf = STR2CSTR(vkey);
   ksiz = RSTRING(vkey)->len;
   curia = crtable[index];
-  if((vsiz = crvsiz(curia, kbuf, ksiz)) == -1) myerror(dpecode);
+  if((vsiz = crvsiz(curia, kbuf, ksiz)) == -1){
+    if(crsltable[index] && dpecode == DP_ENOITEM) return INT2FIX(-1);
+    myerror(dpecode);
+  }
   return INT2FIX(vsiz);
 }
 
@@ -343,7 +369,10 @@ static VALUE rbcriternext(VALUE vself, VALUE vindex){
   FIXNUM_P(vindex);
   if((index = FIX2INT(vindex)) == -1) myerror(DP_EMISC);
   curia = crtable[index];
-  if(!(kbuf = criternext(curia, &ksiz))) myerror(dpecode);
+  if(!(kbuf = criternext(curia, &ksiz))){
+    if(crsltable[index] && dpecode == DP_ENOITEM) return Qnil;
+    myerror(dpecode);
+  }
   vkey = rb_str_new(kbuf, ksiz);
   free(kbuf);
   return vkey;
