@@ -83,6 +83,7 @@ class Depot
   # Method: Called implicitly by the constructor.
   #
   def initialize(name, omode = OREADER, bnum = -1)
+    @silent = false
     MyMutex.synchronize() do
       @index = mod_open(name, omode, bnum)
       @name = name
@@ -115,7 +116,7 @@ class Depot
   ##
   # bool = depot.close()
   # Method: Close the database handle.
-  # The return value is always `true'.
+  # The return value is always true.
   # An exception of `Depot::EANY' or its sub classes is thrown if an error occurs.
   # Because the region of a closed handle is released, it becomes impossible to use the handle.
   # Updating a database is assured to be written when the handle is closed.  If a writer opens
@@ -131,6 +132,19 @@ class Depot
     end
   end
   ##
+  # depot.silent = bool
+  # Method: Set the flag whether to repress frequent exceptions.
+  # The return value is the assigned value.
+  #
+  def silent=(value)
+    @silent = value ? true : false
+    mod_setsilent(@index, silent ? 1 : 0)
+    @silent
+  end
+  def silent
+    @silent
+  end
+  ##
   # bool = depot.put(key, val, dmode)
   # Method: Store a record.
   # `key' specifies a key.  Although it must be an instance of String, binary data is okey.
@@ -139,7 +153,8 @@ class Depot
   # which means the specified value overwrites the existing one, `Depot::DKEEP', which means
   # the existing value is kept, `Depot::DCAT', which means the specified value is concatenated
   # at the end of the existing value.  If it is omitted, `Depot::DOVER' is specified.
-  # The return value is always `true'.
+  # The return value is always true.  However, if the silent flag is true and replace is
+  # cancelled, false is returned instead of exception.
   # An exception of `Depot::EANY' or its sub classes is thrown if an error occurs or replace
   # is cancelled.
   #
@@ -152,7 +167,7 @@ class Depot
   #
   alias store put
   ##
-  # str = (depot[key] = val)
+  # depot[key] = val
   # Method: An alias of `put'.
   #
   alias []= put
@@ -160,7 +175,8 @@ class Depot
   # bool = depot.out(key)
   # Method: Delete a record.
   # `key' specifies a key.  Although it must be an instance of String, binary data is okey.
-  # The return value is always `true'.
+  # The return value is always true.  However, if the silent flag is true and no record
+  # corresponds, false is returned instead of exception.
   # An exception of `Depot::EANY' or its sub classes is thrown if an error occurs or no record
   # corresponds.
   #
@@ -175,7 +191,7 @@ class Depot
   ##
   # bool = depot.clear()
   # Method: Delete all records.
-  # The return value is always `true'.
+  # The return value is always true.
   # An exception of `Depot::EANY' or its sub classes is thrown if an error occurs.
   #
   def clear
@@ -195,7 +211,8 @@ class Depot
   # If it is negative or omitted, the offset is specified as 0.
   # `max' specifies the max size to be read.  If it is negative or omitted, the size to read is
   # unlimited.
-  # The return value is an instance of the value of the corresponding record.
+  # The return value is an instance of the value of the corresponding record.  If the silent flag
+  # is true and no record corresponds, nil is returned instead of exception.
   # An exception of `Depot::EANY' or its sub classes is thrown if an error occurs, no record
   # corresponds, or the size of the value of the corresponding record is less than `max'.
   #
@@ -203,20 +220,41 @@ class Depot
     mod_get(@index, key, start, max)
   end
   ##
-  # str = depot.fetch(key)
-  # Method: An alias of `get'.
+  # str = depot.fetch(key, defval)
+  # Method: Retrieve a record.
+  # `key' specifies a key.  Although it must be an instance of String, binary data is okey.
+  # `defval' specifies the default value used when no record corresponds.  If it is omitted, nil
+  # is specified.
+  # The return value is an instance of the value of the corresponding record, or the default
+  # value if no record corresponds.
+  # An exception of `Depot::EANY' or its sub classes is thrown if an error occurs.
   #
-  alias fetch get
+  def fetch(key, defval = nil)
+    if @silent
+      if val = mod_get(@index, key, 0, -1)
+        val
+      else
+        defval
+      end
+    else
+      begin
+        mod_get(@index, key, 0, -1)
+      rescue ENOITEM
+        defval
+      end
+    end
+  end
   ##
   # str = depot[key]
-  # Method: An alias of `get'.
+  # Method: An alias of `fetch'.
   #
-  alias [] get
+  alias [] fetch
   ##
   # num = depot.vsiz(key)
   # Method: Get the size of the value of a record.
   # `key' specifies a key.  Although it must be an instance of String, binary data is okey.
-  # The return value is the size of the value of the corresponding record.
+  # The return value is the size of the value of the corresponding record.  If the silent flag
+  # is true and no record corresponds, -1 is returned instead of exception.
   # An exception of `Depot::EANY' or its sub classes is thrown if an error occurs or no record
   # corresponds.
   # Because this method does not read the entity of a record, it is faster than `get'.
@@ -227,7 +265,7 @@ class Depot
   ##
   # bool = depot.iterinit()
   # Method: Initialize the iterator of the database handle.
-  # The return value is always `true'.
+  # The return value is always true.
   # An exception of `Depot::EANY' or its sub classes is thrown if an error occurs.
   # The iterator is used in order to access the key of every record stored in a database.
   #
@@ -237,7 +275,8 @@ class Depot
   ##
   # str = depot.iternext()
   # Method: Get the next key of the iterator.
-  # The return value is the value of the next key.
+  # The return value is the value of the next key.  If the silent flag is true and no record
+  # corresponds, nil is returned instead of exception.
   # An exception of `Depot::EANY' or its sub classes is thrown if an error occurs or no record
   # is to be get out of the iterator.
   # It is possible to access every record by iteration of calling this method.  However, it is
@@ -252,7 +291,7 @@ class Depot
   # bool = depot.setalign(align)
   # Method: Set alignment of the database handle.
   # `align' specifies the basic size of alignment.  If it is omitted, alignment is cleared.
-  # The return value is always `true'.
+  # The return value is always true.
   # An exception of `Depot::EANY' or its sub classes is thrown if an error occurs.
   # If alignment is set to a database, the efficiency of overwriting values is improved.
   # The size of alignment is suggested to be average size of the values of the records to be
@@ -269,7 +308,7 @@ class Depot
   # Method: Set the size of the free block pool.
   # `size' specifies the size of the free block pool.  If it is undef, the free block pool is not
   # used.
-  # The return value is always `true'.
+  # The return value is always true.
   # An exception of `Depot::EANY' or its sub classes is thrown if an error occurs.
   # The default size of the free block pool is 16.  If the size is greater, the space efficiency
   # of overwriting values is improved with the time efficiency sacrificed.
@@ -280,7 +319,7 @@ class Depot
   ##
   # bool = depot.sync()
   # Method: Synchronize updating contents with the file and the device.
-  # The return value is always `true'.
+  # The return value is always true.
   # An exception of `Depot::EANY' or its sub classes is thrown if an error occurs.
   # This method is useful when another process uses the connected database file.
   #
@@ -292,7 +331,7 @@ class Depot
   # Method: Optimize the database.
   # `bnum' specifies the number of the elements of the bucket array.  If it is omitted or not
   # more than 0, the default value is specified.
-  # The return value is always `true'.
+  # The return value is always true.
   # An exception of `Depot::EANY' or its sub classes is thrown if an error occurs.
   # In an alternating succession of deleting and storing with overwrite or concatenate,
   # dispensable regions accumulate.  This method is useful to do away with them.
@@ -364,7 +403,7 @@ class Depot
       iterinit()
       while(true)
         begin
-          key = iternext()
+          break unless key = iternext()
           val = get(key)
         rescue ENOITEM
           break
@@ -389,7 +428,7 @@ class Depot
       iterinit()
       while(true)
         begin
-          key = iternext()
+          break unless key = iternext()
         rescue ENOITEM
           break
         end
@@ -408,7 +447,8 @@ class Depot
       iterinit()
       while(true)
         begin
-          val = get(iternext())
+          break unless key = iternext()
+          val = get(key)
         rescue ENOITEM
           break
         end
@@ -466,7 +506,7 @@ class Depot
     MyMutex.synchronize() do
       iterinit()
       while(true)
-        key = iternext()
+        break unless key = iternext()
         (get(key) == val) && break
       end
       iterinit()
