@@ -48,6 +48,7 @@ VALUE cdepoterror_ERMDIR;
 VALUE cdepoterror_EMISC;
 VALUE mdepot;
 DEPOT *dptable[MAXOPEN];
+char dpsltable[MAXOPEN];
 
 
 static void dpinit(void);
@@ -58,6 +59,7 @@ static VALUE rbdpversion(VALUE vself);
 static VALUE rbdperrmsg(VALUE vself, VALUE vecode);
 static VALUE rbdpopen(VALUE vself, VALUE vname, VALUE vomode, VALUE vbnum);
 static VALUE rbdpclose(VALUE vself, VALUE vindex);
+static VALUE rbdpsetsilent(VALUE vself, VALUE vindex, VALUE vvalue);
 static VALUE rbdpput(VALUE vself, VALUE vindex, VALUE vkey, VALUE vval, VALUE vdmode);
 static VALUE rbdpout(VALUE vself, VALUE vindex, VALUE vkey);
 static VALUE rbdpget(VALUE vself, VALUE vindex, VALUE vkey, VALUE vstart, VALUE vmax);
@@ -140,6 +142,7 @@ Init_mod_depot(){
   rb_define_const(mdepot, "DCAT", INT2FIX(DP_DCAT));
   rb_define_module_function(mdepot, "mod_open", rbdpopen, 3);
   rb_define_module_function(mdepot, "mod_close", rbdpclose, 1);
+  rb_define_module_function(mdepot, "mod_setsilent", rbdpsetsilent, 2);
   rb_define_module_function(mdepot, "mod_put", rbdpput, 4);
   rb_define_module_function(mdepot, "mod_out", rbdpout, 2);
   rb_define_module_function(mdepot, "mod_get", rbdpget, 4);
@@ -168,6 +171,7 @@ static void dpinit(void){
   int i;
   for(i = 0; i < MAXOPEN; i++){
     dptable[i] = NULL;
+    dpsltable[i] = 0;
   }
 }
 
@@ -237,6 +241,7 @@ static VALUE rbdpopen(VALUE vself, VALUE vname, VALUE vomode, VALUE vbnum){
   depot = dpopen(name, omode, bnum);
   if(!depot) myerror(dpecode);
   dptable[index] = depot;
+  dpsltable[index] = 0;
   return INT2FIX(index);
 }
 
@@ -253,6 +258,15 @@ static VALUE rbdpclose(VALUE vself, VALUE vindex){
 }
 
 
+static VALUE rbdpsetsilent(VALUE vself, VALUE vindex, VALUE vvalue){
+  int index;
+  FIXNUM_P(vindex);
+  if((index = FIX2INT(vindex)) == -1) myerror(DP_EMISC);
+  dpsltable[index] = FIX2INT(vvalue);
+  return Qnil;
+}
+
+
 static VALUE rbdpput(VALUE vself, VALUE vindex, VALUE vkey, VALUE vval, VALUE vdmode){
   DEPOT *depot;
   const char *kbuf, *vbuf;
@@ -266,7 +280,10 @@ static VALUE rbdpput(VALUE vself, VALUE vindex, VALUE vkey, VALUE vval, VALUE vd
   FIXNUM_P(vdmode);
   dmode = FIX2INT(vdmode);
   depot = dptable[index];
-  if(!dpput(depot, kbuf, ksiz, vbuf, vsiz, dmode)) myerror(dpecode);
+  if(!dpput(depot, kbuf, ksiz, vbuf, vsiz, dmode)){
+    if(dpsltable[index] && dpecode == DP_EKEEP) return Qfalse;
+    myerror(dpecode);
+  }
   return Qtrue;
 }
 
@@ -280,7 +297,10 @@ static VALUE rbdpout(VALUE vself, VALUE vindex, VALUE vkey){
   kbuf = STR2CSTR(vkey);
   ksiz = RSTRING(vkey)->len;
   depot = dptable[index];
-  if(!dpout(depot, kbuf, ksiz)) myerror(dpecode);
+  if(!dpout(depot, kbuf, ksiz)){
+    if(dpsltable[index] && dpecode == DP_ENOITEM) return Qfalse;
+    myerror(dpecode);
+  }
   return Qtrue;
 }
 
@@ -300,7 +320,10 @@ static VALUE rbdpget(VALUE vself, VALUE vindex, VALUE vkey, VALUE vstart, VALUE
   FIXNUM_P(vmax);
   max = FIX2INT(vmax);
   depot = dptable[index];
-  if(!(vbuf = dpget(depot, kbuf, ksiz, start, max, &vsiz))) myerror(dpecode);
+  if(!(vbuf = dpget(depot, kbuf, ksiz, start, max, &vsiz))){
+    if(dpsltable[index] && dpecode == DP_ENOITEM) return Qnil;
+    myerror(dpecode);
+  }
   vval = rb_str_new(vbuf, vsiz);
   free(vbuf);
   return vval;
@@ -316,7 +339,10 @@ static VALUE rbdpvsiz(VALUE vself, VALUE vindex, VALUE vkey){
   kbuf = STR2CSTR(vkey);
   ksiz = RSTRING(vkey)->len;
   depot = dptable[index];
-  if((vsiz = dpvsiz(depot, kbuf, ksiz)) == -1) myerror(dpecode);
+  if((vsiz = dpvsiz(depot, kbuf, ksiz)) == -1){
+    if(dpsltable[index] && dpecode == DP_ENOITEM) return INT2FIX(-1);
+    myerror(dpecode);
+  }
   return INT2FIX(vsiz);
 }
 
@@ -340,7 +366,10 @@ static VALUE rbdpiternext(VALUE vself, VALUE vindex){
   FIXNUM_P(vindex);
   if((index = FIX2INT(vindex)) == -1) myerror(DP_EMISC);
   depot = dptable[index];
-  if(!(kbuf = dpiternext(depot, &ksiz))) myerror(dpecode);
+  if(!(kbuf = dpiternext(depot, &ksiz))){
+    if(dpsltable[index] && dpecode == DP_ENOITEM) return Qnil;
+    myerror(dpecode);
+  }
   vkey = rb_str_new(kbuf, ksiz);
   free(kbuf);
   return vkey;
