@@ -51,6 +51,7 @@ VALUE cvillaerror_EMISC;
 VALUE mvilla;
 ID idcompare;
 VILLA *vltable[MAXOPEN];
+char vlsltable[MAXOPEN];
 
 
 static void vlinit(void);
@@ -62,6 +63,7 @@ static VALUE rbvlversion(VALUE vself);
 static VALUE rbvlerrmsg(VALUE vself, VALUE vecode);
 static VALUE rbvlopen(VALUE vself, VALUE vname, VALUE vomode, VALUE vcmode);
 static VALUE rbvlclose(VALUE vself, VALUE vindex);
+static VALUE rbvlsetsilent(VALUE vself, VALUE vindex, VALUE vvalue);
 static VALUE rbvlput(VALUE vself, VALUE vindex, VALUE vkey, VALUE vval, VALUE vdmode);
 static VALUE rbvlout(VALUE vself, VALUE vindex, VALUE vkey);
 static VALUE rbvlget(VALUE vself, VALUE vindex, VALUE vkey);
@@ -166,6 +168,7 @@ Init_mod_villa(){
   rb_define_const(mvilla, "CPAFTER", INT2FIX(VL_CPAFTER));
   rb_define_module_function(mvilla, "mod_open", rbvlopen, 3);
   rb_define_module_function(mvilla, "mod_close", rbvlclose, 1);
+  rb_define_module_function(mvilla, "mod_setsilent", rbvlsetsilent, 2);
   rb_define_module_function(mvilla, "mod_put", rbvlput, 4);
   rb_define_module_function(mvilla, "mod_out", rbvlout, 2);
   rb_define_module_function(mvilla, "mod_get", rbvlget, 2);
@@ -230,6 +233,7 @@ static void vlinit(void){
   int i;
   for(i = 0; i < MAXOPEN; i++){
     vltable[i] = NULL;
+    vlsltable[i] = 0;
   }
 }
 
@@ -316,6 +320,7 @@ static VALUE rbvlopen(VALUE vself, VALUE vname, VALUE vomode, VALUE vcmode){
   villa = vlopen(name, omode, cmp);
   if(!villa) myerror(dpecode);
   vltable[index] = villa;
+  vlsltable[index] = 0;
   return INT2FIX(index);
 }
 
@@ -332,6 +337,15 @@ static VALUE rbvlclose(VALUE vself, VALUE vindex){
 }
 
 
+static VALUE rbvlsetsilent(VALUE vself, VALUE vindex, VALUE vvalue){
+  int index;
+  FIXNUM_P(vindex);
+  if((index = FIX2INT(vindex)) == -1) myerror(DP_EMISC);
+  vlsltable[index] = FIX2INT(vvalue);
+  return Qnil;
+}
+
+
 static VALUE rbvlput(VALUE vself, VALUE vindex, VALUE vkey, VALUE vval, VALUE vdmode){
   VILLA *villa;
   const char *kbuf, *vbuf;
@@ -345,7 +359,10 @@ static VALUE rbvlput(VALUE vself, VALUE vindex, VALUE vkey, VALUE vval, VALUE vd
   FIXNUM_P(vdmode);
   dmode = FIX2INT(vdmode);
   villa = vltable[index];
-  if(!vlput(villa, kbuf, ksiz, vbuf, vsiz, dmode)) myerror(dpecode);
+  if(!vlput(villa, kbuf, ksiz, vbuf, vsiz, dmode)){
+    if(vlsltable[index] && dpecode == DP_EKEEP) return Qfalse;
+    myerror(dpecode);
+  }
   return Qtrue;
 }
 
@@ -359,7 +376,10 @@ static VALUE rbvlout(VALUE vself, VALUE vindex, VALUE vkey){
   kbuf = STR2CSTR(vkey);
   ksiz = RSTRING(vkey)->len;
   villa = vltable[index];
-  if(!vlout(villa, kbuf, ksiz)) myerror(dpecode);
+  if(!vlout(villa, kbuf, ksiz)){
+    if(vlsltable[index] && dpecode == DP_ENOITEM) return Qfalse;
+    myerror(dpecode);
+  }
   return Qtrue;
 }
 
@@ -374,7 +394,10 @@ static VALUE rbvlget(VALUE vself, VALUE vindex, VALUE vkey){
   kbuf = STR2CSTR(vkey);
   ksiz = RSTRING(vkey)->len;
   villa = vltable[index];
-  if(!(vbuf = vlget(villa, kbuf, ksiz, &vsiz))) myerror(dpecode);
+  if(!(vbuf = vlget(villa, kbuf, ksiz, &vsiz))){
+    if(vlsltable[index] && dpecode == DP_ENOITEM) return Qnil;
+    myerror(dpecode);
+  }
   vval = rb_str_new(vbuf, vsiz);
   return vval;
 }
@@ -389,7 +412,10 @@ static VALUE rbvlvsiz(VALUE vself, VALUE vindex, VALUE vkey){
   kbuf = STR2CSTR(vkey);
   ksiz = RSTRING(vkey)->len;
   villa = vltable[index];
-  vsiz = vlvsiz(villa, kbuf, ksiz);
+  if((vsiz = vlvsiz(villa, kbuf, ksiz)) == -1){
+    if(vlsltable[index] && dpecode == DP_ENOITEM) return INT2FIX(-1);
+    myerror(dpecode);
+  }
   return INT2FIX(vsiz);
 }
 
@@ -414,7 +440,10 @@ static VALUE rbvlcurfirst(VALUE vself, VALUE vindex){
   FIXNUM_P(vindex);
   if((index = FIX2INT(vindex)) == -1) myerror(DP_EMISC);
   villa = vltable[index];
-  if(!vlcurfirst(villa)) myerror(dpecode);
+  if(!vlcurfirst(villa)){
+    if(vlsltable[index] && dpecode == DP_ENOITEM) return Qfalse;
+    myerror(dpecode);
+  }
   return Qtrue;
 }
 
@@ -425,7 +454,10 @@ static VALUE rbvlcurlast(VALUE vself, VALUE vindex){
   FIXNUM_P(vindex);
   if((index = FIX2INT(vindex)) == -1) myerror(DP_EMISC);
   villa = vltable[index];
-  if(!vlcurlast(villa)) myerror(dpecode);
+  if(!vlcurlast(villa)){
+    if(vlsltable[index] && dpecode == DP_ENOITEM) return Qfalse;
+    myerror(dpecode);
+  }
   return Qtrue;
 }
 
@@ -436,7 +468,10 @@ static VALUE rbvlcurprev(VALUE vself, VALUE vindex){
   FIXNUM_P(vindex);
   if((index = FIX2INT(vindex)) == -1) myerror(DP_EMISC);
   villa = vltable[index];
-  if(!vlcurprev(villa)) myerror(dpecode);
+  if(!vlcurprev(villa)){
+    if(vlsltable[index] && dpecode == DP_ENOITEM) return Qfalse;
+    myerror(dpecode);
+  }
   return Qtrue;
 }
 
@@ -447,7 +482,10 @@ static VALUE rbvlcurnext(VALUE vself, VALUE vindex){
   FIXNUM_P(vindex);
   if((index = FIX2INT(vindex)) == -1) myerror(DP_EMISC);
   villa = vltable[index];
-  if(!vlcurnext(villa)) myerror(dpecode);
+  if(!vlcurnext(villa)){
+    if(vlsltable[index] && dpecode == DP_ENOITEM) return Qfalse;
+    myerror(dpecode);
+  }
   return Qtrue;
 }
 
@@ -464,7 +502,10 @@ static VALUE rbvlcurjump(VALUE vself, VALUE vindex, VALUE vkey, VALUE vjmode){
   FIXNUM_P(vjmode);
   jmode = FIX2INT(vjmode);
   villa = vltable[index];
-  if(!vlcurjump(villa, kbuf, ksiz, jmode)) myerror(dpecode);
+  if(!vlcurjump(villa, kbuf, ksiz, jmode)){
+    if(vlsltable[index] && dpecode == DP_ENOITEM) return Qfalse;
+    myerror(dpecode);
+  }
   return Qtrue;
 }
 
@@ -478,7 +519,10 @@ static VALUE rbvlcurkey(VALUE vself, VALUE vindex){
   FIXNUM_P(vindex);
   if((index = FIX2INT(vindex)) == -1) myerror(DP_EMISC);
   villa = vltable[index];
-  if(!(kbuf = vlcurkeycache(villa, &ksiz))) myerror(dpecode);
+  if(!(kbuf = vlcurkeycache(villa, &ksiz))){
+    if(vlsltable[index] && dpecode == DP_ENOITEM) return Qnil;
+    myerror(dpecode);
+  }
   vkey = rb_str_new(kbuf, ksiz);
   return vkey;
 }
@@ -492,7 +536,10 @@ static VALUE rbvlcurval(VALUE vself, VALUE vindex){
   FIXNUM_P(vindex);
   if((index = FIX2INT(vindex)) == -1) myerror(DP_EMISC);
   villa = vltable[index];
-  if(!(vbuf = vlcurvalcache(villa, &vsiz))) myerror(dpecode);
+  if(!(vbuf = vlcurvalcache(villa, &vsiz))){
+    if(vlsltable[index] && dpecode == DP_ENOITEM) return Qnil;
+    myerror(dpecode);
+  }
   vval = rb_str_new(vbuf, vsiz);
   return vval;
 }
@@ -509,7 +556,10 @@ static VALUE rbvlcurput(VALUE vself, VALUE vindex, VALUE vval, VALUE vcpmode){
   FIXNUM_P(vcpmode);
   cpmode = FIX2INT(vcpmode);
   villa = vltable[index];
-  if(!vlcurput(villa, vbuf, vsiz, cpmode)) myerror(dpecode);
+  if(!vlcurput(villa, vbuf, vsiz, cpmode)){
+    if(vlsltable[index] && dpecode == DP_ENOITEM) return Qfalse;
+    myerror(dpecode);
+  }
   return Qtrue;
 }
 
@@ -520,7 +570,10 @@ static VALUE rbvlcurout(VALUE vself, VALUE vindex){
   FIXNUM_P(vindex);
   if((index = FIX2INT(vindex)) == -1) myerror(DP_EMISC);
   villa = vltable[index];
-  if(!vlcurout(villa)) myerror(dpecode);
+  if(!vlcurout(villa)){
+    if(vlsltable[index] && dpecode == DP_ENOITEM) return Qfalse;
+    myerror(dpecode);
+  }
   return Qtrue;
 }
 
