@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Implementation of Villa
- *                                                      Copyright (C) 2000-2006 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2007 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -45,20 +45,20 @@
 /* set a buffer for a variable length number */
 #define VL_SETVNUMBUF(VL_len, VL_buf, VL_num) \
   do { \
-    int _VL_num = VL_num; \
-    div_t VL_d; \
+    int _VL_num; \
+    _VL_num = VL_num; \
     if(_VL_num == 0){ \
       ((signed char *)(VL_buf))[0] = 0; \
       (VL_len) = 1; \
     } else { \
       (VL_len) = 0; \
       while(_VL_num > 0){ \
-        VL_d = div(_VL_num, 128); \
-        _VL_num = VL_d.quot; \
+        int _VL_rem = _VL_num & 0x7f; \
+        _VL_num >>= 7; \
         if(_VL_num > 0){ \
-          ((signed char *)(VL_buf))[(VL_len)] = -VL_d.rem - 1; \
+          ((signed char *)(VL_buf))[(VL_len)] = -_VL_rem - 1; \
         } else { \
-          ((signed char *)(VL_buf))[(VL_len)] = VL_d.rem; \
+          ((signed char *)(VL_buf))[(VL_len)] = _VL_rem; \
         } \
         (VL_len)++; \
       } \
@@ -2171,7 +2171,7 @@ static VLLEAF *vlgethistleaf(VILLA *villa, const char *kbuf, int ksiz){
   VLLEAF *leaf;
   VLREC *recp;
   int ln, rv;
-  assert(villa);
+  assert(villa && kbuf && ksiz >= 0);
   if(!(leaf = vlleafload(villa, villa->hleaf))) return NULL;
   if((ln = CB_LISTNUM(leaf->recs)) < 2) return NULL;
   recp = (VLREC *)CB_LISTVAL(leaf->recs, 0);
@@ -2341,7 +2341,7 @@ static VLLEAF *vlleafdivide(VILLA *villa, VLLEAF *leaf){
 
 /* Create a new node.
    `villa' specifies a database handle.
-   `id' specifies the ID number of the node.
+   `heir' specifies the ID of the child before the first index.
    The return value is a handle of the node. */
 static VLNODE *vlnodenew(VILLA *villa, int heir){
   VLNODE nent;
